# Created for BADS 2018
# see README.md for details
# Python 3

"""
The quicksort module provides methods for sorting an array and selecting
the ith smallest element in an array using quicksort.

For additional documentation, see Section 2.3 of Algorithms, 4th Edition
by Robert Sedgewick and Kevin Wayne.

:original author: Robert Sedgewick and Kevin Wayne
:original java code: https://algs4.cs.princeton.edu/23quicksort/Quick.java.html
"""

import sys

from itu.algs4.stdlib import stdio, stdrandom


def sort(array):
    """
    Rearranges the array in ascending order, using the natural order
    """
    stdrandom.shuffle(array)
    _sort(array, 0, len(array) - 1)

# quicksort the subarray from array[lo] to array[hi]
def _sort(array, lo, hi):
    if hi <= lo:
        return
    j = _partition(array, lo, hi)
    _sort(array, lo, j-1)
    _sort(array, j+1, hi)

# partition the subarray array[lo..hi] so that
# array[lo..j-1] <= array[j] <= array[j+1..hi]
# and return the index j
def _partition(array, lo, hi):
    i = lo
    j = hi + 1
    v = array[lo]
    while True:

        while array[i+1] < v:
            i += 1
            if i == hi:
                break
        i += 1

        # find item on hi to swap
        while v < array[j-1]:
            j -= 1
            if j == lo:
                break
        j -= 1

        # check if pointers cross
        if i >= j:
            break

        _exch(array, i, j)

    # put partitioning item v at a[j]
    _exch(array, lo, j)

    # now array[lo .. j-1] <= a[j] <= a[j+1 .. hi]
    return j

def select(array, k):
    """
    Rearranges the array so that array[k] contains the kth smalles key;
    array[0] through array[k-1] are less than (or equal to) array[k];
    and array[k+1] through array[n-1] are greather than (or equal to)
    array[k]

    :param array: the array
    :param k: the rank of the key
    :return: the key of rank k
    """
    stdrandom.shuffle(array)
    lo = 0
    hi = len(array) - 1
    while hi > lo:
        i = _partition(array, lo, hi)
        if i > k:
            hi = i - 1
        elif i < k:
            lo = i + 1
        else:
            return array[i]
    return array[lo]

# exchange array[i] and array[j]
def _exch(array, i, j):
    swap = array[i]
    array[i] = array[j]
    array[j] = swap


###########################################################
##### Check if array is sorted - useful for debugging #####
###########################################################

def is_sorted(array):
    return _is_sorted(array, 0, len(array) - 1)

def _is_sorted(array, lo, hi):
    for i in range(lo + 1, hi + 1):
        if array[i] < array[i-1]:
            return False
    return True

# print array to standard output
def show(array):
    stdio.write(" ".join(array))


if __name__ == "__main__":
    array = stdio.readAllStrings()
    sort(array)
    assert is_sorted(array)
    show(array)

    # shuffle
    stdrandom.shuffle(array)

    # display results again using select
    print()
    for i in range(0, len(array)):
        ith = str(select(array, i))
        stdio.writeln(ith)



# Created for BADS 2018
# see README.md for details
# This is python3 

from itu.algs4.fundamentals.stack import Stack


class DepthFirstPaths:
    """
    The  DepthFirstPaths class represents a data type for finding
    paths from a source vertex s to every other vertex
    in an undirected graph.

    This implementation uses depth-first search.
    The constructor takes time proportional to V + E,
    where V is the number of vertices and E is the number of edges.
    Each call to hasPathTo(int) takes constant time
    each call to pathTo(int) takes time proportional to the length
    of the path.
    It uses extra space (not including the graph) proportional to V.
    """

    def __init__(self, G, s):
        """
        Computes a path between s and every other vertex in graph G.

        :param G: the graph
        :param s: the source vertex
        :raises ValueError: unless 0 <= s < V     
        """
        self._marked = [False] * G.V()  # Has dfs been called for this vertex?
        self._edgeTo = [0] * G.V()      # last vertex on known path to this vertex
        self._s = s                     # source        
        self._validateVertex(s)
        self._dfs(G, s)

    def _dfs(self, G, v):
        # depth first search from v
        self._marked[v] = True
        for w in G.adj(v):
            if not self._marked[w]:
                self._edgeTo[w] = v
                self._dfs(G, w)

    def has_path_to(self, v):
        """
        Is there a path between the source vertex s and vertex v?
        :param v: the vertex
        :returns: true if there is a path, false otherwise
        :raises ValueError: unless 0 <= v < V
        """
        self._validateVertex(v)
        return self._marked[v]

    def path_to(self, v):
        """
        Returns a path between the source vertex s and vertex v, or
        None if no such path.
        :param v: the vertex
        :returns: the sequence of vertices on a path between the source vertex
                   s and vertex v, as an Iterable
        :raises ValueError: unless 0 <= v < V
        """
        self._validateVertex(v)
        if not self.has_path_to(v): return None
        path = Stack()
        w = v
        while w != self._s:
            path.push(w)
            w = self._edgeTo[w]
        path.push(self._s)

        return path

    def _validateVertex(self, v):
        # throw an ValueError unless 0 <= v < V
        V = len(self._marked)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))

if __name__ == "__main__":
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.graph import Graph
    from itu.algs4.stdlib.instream import InStream
    import sys

    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    s = int(sys.argv[2])
    dfs = DepthFirstPaths(G, s)

    for v in range(G.V()):
        if dfs.has_path_to(v):
            stdio.writef("%d to %d:  ", s, v)
            for x in dfs.path_to(v):
                if x == s: stdio.write(x)
                else:      stdio.writef("-%i", x)            
            stdio.writeln()
        else:
            stdio.writef("%d to %d:  not connected\n", s, v)            



# Created for BADS 2018
# See README.md for details
# Python 3
from itu.algs4.errors.errors import IllegalArgumentException, NoSuchElementException


"""
 The ST class represents an ordered symbol table of generic
 key-value pairs.
"""


class ST:
    def __init__(self):
        self._st = dict()

    # Returns the value associated with the given key in this symbol table.
    def get(self, key):
        if key is None:
            raise IllegalArgumentException("called get() with None key")
        return self._st.get(key)

    # Inserts the specified key-value pair into the symbol table, overwriting the old
    # value with the new value if the symbol table already contains the specified key.
    # Deletes the specified key (and its associated value) from this symbol table
    # if the specified value is None.
    def put(self, key, val):
        if key is None:
            raise IllegalArgumentException("called put() with None key")
        if val is None:
            self._st.pop(key, None)
        else:
            self._st[key] = val

    # Removes the specified key and its associated value from this symbol table
    # (if the key is in this symbol table).
    def delete(self, key):
        if key is None:
            raise IllegalArgumentException("called delete() with None key")
        self._st.pop(key, None)

    # Returns true if this symbol table contain the given key.
    def contains(self, key):
        if key is None:
            raise IllegalArgumentException("called contains() with None key")
        return key in self._st

    # Returns the number of key-value pairs in this symbol table.
    def size(self):
        return len(self._st)

    def __len__(self):
        return self.size()

    # Returns true if this symbol table is empty.
    def is_empty(self):
        return self.size() == 0

    # Returns all keys in this symbol table.
    # To iterate over all of the keys in the symbol table named st,
    # use the foreach notation: for key in st.keys() .
    def keys(self):
        return self._st.keys()

    def __iter__(self):
        for k in self.keys():
            yield k

    # Returns the smallest key in this symbol table.
    def min(self):
        if self.is_empty():
            raise NoSuchElementException("called min() with empty symbol table")
        return min(self._st)

    # Returns the largest key in this symbol table.
    def max(self):
        if self.is_empty():
            raise NoSuchElementException("called max() with empty symbol table")
        return max(self._st)

    # Returns the smallest key in this symbol table greater than or equal to key.
    def ceiling(self, key):
        if key is None:
            raise IllegalArgumentException("called ceiling() with None key")
        keys = self.keys()
        ceiling = None
        for k in keys:
            if (ceiling is None and k >= key) or (
                ceiling is not None and k >= key and k < ceiling
            ):
                ceiling = k
        if ceiling is None:
            raise NoSuchElementException("all keys are less than " + str(key))
        return ceiling

    # Returns the largest key in this symbol table less than or equal to key.
    def floor(self, key):
        if key is None:
            raise IllegalArgumentException("called floor() with None key")
        keys = self.keys()
        floor = None
        for k in keys:
            if (floor is None and k <= key) or (
                floor is not None and k <= key and k > floor
            ):
                floor = k
        if floor is None:
            raise NoSuchElementException("all keys are greater than " + str(key))
        return floor



# Created for BADS 2018
# See README.md for details
# This is python3
import struct
import sys


"""
Binary output. This class provides methods for converting
some primitive type variables (boolean, byte, char, and int)
to sequences of bits and writing them
to an output stream.
The output stream can be standard output or another outputstream.
Uses big-endian (most-significant byte first).

The client must flush() the output stream when finished writing bits.

The client should not intermix calls to BinaryOut with calls to stdout;
otherwise unexpected behavior will result.
"""
class BinaryOut:
	def __init__(self, os=sys.stdout):
		"""
		Initializes a binary output stream from a
		specified output stream. Defaults to stdin.

		:param os: the output streamt to write to.
		"""
		self.out = os.buffer
		self.buffer = 0 #8-bit buffer of bits to write out
		self.n = 0 #number of bits used in buffer

	def _writeBit(self, x):
		self.buffer <<= 1
		if(x):
			self.buffer |=1
		self.n += 1
		if(self.n == 8):
			self._clearBuffer()

	def _writeByte(self, x):
		assert x >= 0 and x < 256
		#optimized if byte-alligned
		if(self.n == 0):
			self.out.write(struct.pack('B',x))
			return
		#otherwise write one bit at a time
		for i in range(0,8):
			bit = ((x >> (8 - i - 1)) & 1) == 1
			self._writeBit(bit)

	def _clearBuffer(self):
		if(self.n == 0):
			return
		if(self.n > 0):
			self.buffer <<= (8-self.n)
		self.out.write(self.buffer.to_bytes(1,'big'))
		self.n = 0
		self.buffer = 0

	def flush(self):
		self._clearBuffer()
		self.out.flush()

	def close(self):
		self.flush()
		self.out.close()

	def write_bool(self,x):
		self._writeBit(x)

	def write_byte(self,x):
		self._writeByte(x & 0xff)

	def write_int(self,x):
		self._writeByte(((x >> 24)& 0xff))
		self._writeByte(((x >> 16)& 0xff))
		self._writeByte(((x >> 8)& 0xff))
		self._writeByte(((x >> 0)& 0xff))

	def write_char(self, x):
		if(ord(x)<0 or ord(x)>=256):
			raise ValueError("Illegal 8-bit char = {}".format(x))
		self._writeByte(ord(x))

	def write_string(self, s):
		for i in s:
			self.write_char(s[i])
def main():
	out = BinaryOut()
	for i in sys.argv[1]:
		out.write_char(i)
	out.close()
if __name__ == '__main__':
	main()



# Created for BADS 2018
# See README.md for details
# Python 3

"""
This module implements the directed cycle algorithm described in 
Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne. This version
works for both weighted and unweighted directed graphs, due to Python's duck-typing.

For more
information, see chapter 4.2 of the book.
"""

import sys

from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.digraph import Digraph
from itu.algs4.stdlib.instream import InStream


class DirectedCycle:
    """
    The DirectedCycle class represents a data type for determining whether a 
    digraph has a directed cycle. The hasCycle operation determines whether the 
    digraph has a directed cycle and, and of so, the cycle operation returns one.

    This implementation uses depth-first search. The constructor takes time proportional 
    to V + E (in the worst case), where V is the number of vertices and E is the 
    number of edges. Afterwards, the hasCycle operation takes constant time; the 
    cycle operation takes time proportional to the length of the cycle.

    See Topological to compute a topological order if the digraph is acyclic.

    For additional documentation, see Section 4.2 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """    
    
    def __init__(self, digraph):
        """
        Determines whether the digraph has a directed cycle and, if so,
        finds such a cycle.
        
        :digraph: the digraph
        """
        self._cycle = None
        self._on_stack = [False]*digraph.V()
        self._edge_to  = [0]*digraph.V()
        self._marked = [False]*digraph.V()
        for v in range(digraph.V()):
            if not self._marked[v]:
                self._dfs(digraph, v) 
    
    # check that algorithm computes either the topological order or finds a directed cycle
    def _dfs(self, digraph, v):
        self._on_stack[v] = True
        self._marked[v] = True
        for w in digraph.adj(v):
            # short circuit if directed cycle found
            if self.has_cycle():
                return
            # found new vertex, so recur
            elif not self._marked[w]:
                self._edge_to[w] = v
                self._dfs(digraph, w)
            # trace back directed cycle
            elif self._on_stack[w]:
                self._cycle = Stack()
                x = v
                while x != w:
                    self._cycle.push(x)
                    x = self._edge_to[x]
                self._cycle.push(w)
                self._cycle.push(v)
                
        self._on_stack[v] = False
    
    def has_cycle(self):
        """
        Does the digraph have a directed cycle?
        
        :returns: true if there is a cycle, false otherwise
        """
        return self._cycle != None
    
    def cycle(self):
        """
        Returns a directed cycle if the digraph has a directed cycle, and null otherwise.
        
        :returns: a directed cycle (as an iterable) if the digraph has a directed cycle, and null otherwise
        """
        return self._cycle



if __name__ == '__main__':
    # Create stream from file or the standard input,
    # depending on whether a file name was passed.
    stream = sys.argv[1] if len(sys.argv) > 1 else None
    
    d = Digraph.from_stream(InStream(stream))
 
    cyc = DirectedCycle(d)
    print(cyc.cycle())



# Created for BADS 2018
# See README.md for details
# This is python3 
"""
The Quick3String module provides functions for sorting an
array of strings using 3-way radix quicksort
"""
import sys


def sort(a):
	"""
	Rearranges the array of strings in ascending order.
	:param a: the array to be sorted.
	"""
	_sort(a, 0, len(a)-1, 0)
	
def _sort(a, lo, hi, d):
	if(hi <= lo):
		return
	lt = lo
	gt = hi
	v = _char_at(a[lo], d)
	i = lo + 1
	while (i <= gt):
		t = _char_at(a[i], d)
		if (t < v):
			_exch(a, lt, i)
			lt += 1
			i += 1
		elif (t > v):
			_exch(a, i, gt)
			gt -= 1
		else:
			i += 1
	_sort(a, lo, lt-1, d)
	if (v >= 0):
		_sort(a, lt, gt, d+1)
	_sort(a, gt+1, hi, d)
	
def _char_at(s, d):
	if(d < len(s)):
		return ord(s[d])
	else:
		return -1
	
def _show(a):
	for item in a:
		print(item)
		
def is_sorted(a):
	"""
	Returns true if a is sorted.
	:param a: the array to be checked.
	:returns: True if a is sorted.
	"""
	for i in range(1,len(a)):
		if _less(a[i], a[i-1]):
			return False
	return True

def _less(v, w):
	return v < w

def _exch(a, i, j):
	t = a[i]
	a[i] = a[j]
	a[j] = t
	
def main():
	"""
	Reads in a sequence of fixed-length strings from standard input;
	3-way radix quicksorts them;
	and prints them to standard output in ascending order.
	"""
	a = sys.argv[1:]
	sort(a)
	assert is_sorted(a)
	_show(a)
	
if __name__ == "__main__":
	main()



# Created for BADS 2018
# See README.md for details
# This is python3 
import sys

from itu.algs4.fundamentals.bag import Bag
from itu.algs4.graphs.digraph import Digraph
from itu.algs4.stdlib.instream import InStream


class DirectedDFS:
	"""
	The DirectedDFS class represents a data type for
	determining the vertices reachable from a given source vertex s
	(or a set of source vertices) in a digraph. For versions that find the paths,
	see DepthFirstDirectedPaths and BreadthFirstDirectedPaths.

	This implementation uses depth-first search.
	The constructor takes time proportional to V + E (in the worst case),
	where V is the number of vertices and E is the number of edges.

	For additional documentation, see Section 4.2 of Algorithms,
	4th Edition by Robert Sedgewick and Kevin Wayne.
	"""
	def __init__(self, G, *s):
		"""
		Computes the vertices in digraph G that are
		reachable from the source vertex s.

		:param G: the digraph
		:param s: the source vertex/vertices
		:raises ValueError: unless 0 <= s_ < V for every s_ in s
		"""
		self.marked = [False for i in range(0,G.V())]
		self.reachables = 0
		for s_ in s:
			self._validate_vertex(s_)
			self._dfs(G, s_)

	def _dfs(self, G, v):
		self.reachables += 1
		self.marked[v] = True
		for w in G.adj(v):
			if(not self.marked[w]):
				self._dfs(G, w)

	def is_marked(self, v):
		"""
		Is there a directed path from the source vertex
		and vertex v?

		:param v: the vertex
		:returns: True if there is a directed
		"""
		self._validate_vertex(v)
		return self.marked[v]

	def count(self):
		"""
		Returns the number of vertices reachable from the source vertex
		(or source vertices)
		:returns: the number of vertices reachable from the source vertex
		(or source vertices)
		"""
		return self.reachables

	def _validate_vertex(self, v):
		# Raise a ValueError unless 0 <= v < V
		V = len(self.marked)
		if(v < 0 or v >= V):
			raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))

def main():
	"""
	Unit tests the DirectedDFS data type.
	"""
	G = Digraph.from_stream(InStream(None))
	sources = Bag()
	for i in range(1,len(sys.argv)):
		s = int(sys.argv[i])
		sources.add(s)

	dfs = DirectedDFS(G, *sources)
	print("Reachable vertices:")
	for v in range(0, G.V()):
		if(dfs.is_marked(v)):
			print(v,end=" ")
	print()
if __name__ == '__main__':
	main()



# Created for BADS 2018
# See README.md for details
# Python 3
import sys

from itu.algs4.errors.errors import (
    IllegalArgumentException,
    UnsupportedOperationException,
)
from itu.algs4.fundamentals.queue import Queue
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.directed_edge import DirectedEdge
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.graphs.edge_weighted_directed_cycle import EdgeWeightedDirectedCycle
from itu.algs4.stdlib import stdio
from itu.algs4.stdlib.instream import InStream

try:
    q = Queue()
    q.enqueue(1)
except AttributeError:
    print('ERROR - Could not import itu.algs4 queue')
    sys.exit(1)

#  Execution:    python BellmanFordSP.py filename.txt s
#  Data files:   https://algs4.cs.princeton.edu/44sp/tinyEWDn.txt
#                https://algs4.cs.princeton.edu/44sp/mediumEWDnc.txt
#
#  Bellman-Ford shortest path algorithm. Computes the shortest path tree in
#  edge-weighted digraph G from vertex s, or finds a negative cost cycle
#  reachable from s.

"""
 The BellmanFordSP class represents a data type for solving the
 single-source shortest paths problem in edge-weighted digraphs with
 no negative cycles. 
 The edge weights can be positive, negative, or zero.
 This class finds either a shortest path from the source vertex s
 to every other vertex or a negative cycle reachable from the source vertex.
 This implementation uses the Bellman-Ford-Moore algorithm.
 The constructor takes time proportional to V (V + E)
 in the worst case, where V is the number of vertices and E
 is the number of edges.
 Each call to distTo(int) and hasPathTo(int),
 hasNegativeCycle takes constant time;
 each call to pathTo(int) and negativeCycle()
 takes time proportional to length of the path returned.
"""

class BellmanFordSP:
    # Computes a shortest paths tree from s to every other vertex in
    # the edge-weighted digraph G.
    # @param G the acyclic digraph
    # @param s the source vertex
    # @throws IllegalArgumentException unless 0 <= s < V
    def __init__(self, G, s):
        self._distTo = [sys.float_info.max] * G.V()  #distTo[v] = distance  of shortest s->v path
        self._edgeTo = [None] * G.V()                #edgeTo[v] = last edge on shortest s->v path
        self._onQueue = [False] * G.V()              #onQueue[v] = is v currently on the queue?
        self._queue = Queue()                        #queue of vertices to relax
        self._cost = 0                               #number of calls to relax()
        self._cycle = None                           #negative cycle (or None if no such cycle)
        
        #Bellman-Ford algorithm
        self._distTo[s] = 0.0
        self._queue.enqueue(s)
        self._onQueue[s] = True
        while not self._queue.is_empty() and not self.has_negative_cycle():
            v = self._queue.dequeue()
            self._onQueue[v] = False
            self._relax(G, v)
        assert self._check(G, s)

    #relax vertex v and put other endpoints on queue if changed
    def _relax(self, G, v):
        for e in G.adj(v):
            w = e.to_vertex()
            if self._distTo[w] > self._distTo[v] + e.weight():
                self._distTo[w] = self._distTo[v] + e.weight()
                self._edgeTo[w] = e
                if not self._onQueue[w]:
                    self._queue.enqueue(w)
                    self._onQueue[w] = True
            if self._cost % G.V() == 0:
                self._find_negative_cycle()
                if self.has_negative_cycle():
                    return  #found a negative cycle
            self._cost += 1
        
    # Is there a negative cycle reachable from the source vertex s?
    # @return true if there is a negative cycle reachable from the
    #    source vertex s, and false otherwise
    def has_negative_cycle(self):
        return self._cycle is not None

    # Returns a negative cycle reachable from the source vertex s, or None
    # if there is no such cycle.
    # @return a negative cycle reachable from the soruce vertex s 
    #    as an iterable of edges, and None if there is no such cycle
    def negative_cycle(self):
        return self._cycle

    #by finding a cycle in predecessor graph
    def _find_negative_cycle(self):
        V = len(self._edgeTo)
        spt = EdgeWeightedDigraph(V)
        for v in range(V):
            if self._edgeTo[v] is not None:
                spt.add_edge(self._edgeTo[v])

        finder = EdgeWeightedDirectedCycle(spt)
        self._cycle = finder.cycle()

    # Returns the length of a shortest path from the source vertex s to vertex v.
    # @param  v the destination vertex
    # @return the length of a shortest path from the source vertex s to vertex v;
    #         sys.float_info.max if no such path
    # @throws UnsupportedOperationException if there is a negative cost cycle reachable
    #         from the source vertex s
    # @throws IllegalArgumentException unless 0 <= v < V
    def dist_to(self, v):
        self._validate_vertex(v)
        if self.has_negative_cycle():
            raise UnsupportedOperationException("Negative cost cycle exists")
        return self._distTo[v]
    
    # Is there a path from the source s to vertex v?
    # @param  v the destination vertex
    # @return true if there is a path from the source vertex
    #         s to vertex v, and false otherwise
    # @throws IllegalArgumentException unless 0 <= v < V
    def has_path_to(self, v):
        self._validate_vertex(v)
        return self._distTo[v] < sys.float_info.max

    # Returns a shortest path from the source s to vertex v.
    # @param  v the destination vertex
    # @return a shortest path from the source s to vertex v
    #         as an iterable of edges, and None if no such path
    # @throws UnsupportedOperationException if there is a negative cost cycle reachable
    #         from the source vertex s
    # @throws IllegalArgumentException unless 0 <= v < V
    def path_to(self, v):
        self._validate_vertex(v)
        if self.has_negative_cycle():
            raise UnsupportedOperationException("Negative cost cycle exists")
        if not self.has_path_to(v): return None
        path = Stack()
        e = self._edgeTo[v]
        while e is not None:
            path.push(e)
            e = self._edgeTo[e.from_vertex()]
        return path

    #check optimality conditions: either 
    #(i) there exists a negative cycle reacheable from s
    #    or 
    #(ii)  for all edges e = v->w:            distTo[w] <= distTo[v] + e.weight()
    #(ii') for all edges e = v->w on the SPT: distTo[w] == distTo[v] + e.weight()
    def _check(self, G, s):

        #has a negative cycle
        if self.has_negative_cycle():
            weight = 0.0
            for e in self.negative_cycle():
                weight += e.weight()
            if weight >= 0.0:
                print("error: weight of negative cycle = {}".format(weight))
                return False

        #no negative cycle reachable from source
        else:

            #check that distTo[v] and edgeTo[v] are consistent
            if self._distTo[s] != 0.0 or self._edgeTo[s] is not None:
                print("distanceTo[s] and edgeTo[s] inconsistent")
                return False
            
            for v in range(G.V()):
                if v == s: continue
                if self._edgeTo[v] is None and self._distTo[v] != sys.float_info.max:
                    print("distTo[] and edgeTo[] inconsistent")
                    return False

            #check that all edges e = v->w satisfy distTo[w] <= distTo[v] + e.weight()
            for v in range(G.V()):
                for e in G.adj(v):
                    w = e.to_vertex()
                    if self._distTo[v] + e.weight() < self._distTo[w]:
                        print("edge {} not relaxed".format(e))
                        return False

            #check that all edges e = v->w on SPT satisfy distTo[w] == distTo[v] + e.weight()
            for w in range(G.V()):
                if self._edgeTo[w] is None: continue
                e = self._edgeTo[w]
                v = e.from_vertex()
                if w != e.to_vertex(): return False
                if self._distTo[v] + e.weight() != self._distTo[w]:
                    print("edge {} on shortest path not tight".format(e))
                    return False

        print("Satisfies optimality conditions")
        print()
        return True

    #raise an IllegalArgumentException unless 0 <= v < V
    def _validate_vertex(self, v):
        V = len(self._distTo)
        if v < 0 or v >= V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v, V-1))
    

def main(args):
    stream = InStream(args[0])
    s = int(args[1])
    G = EdgeWeightedDigraph.from_stream(stream)
    sp = BellmanFordSP(G, s)

    #print negative cycle
    if sp.has_negative_cycle():
        for e in sp.negative_cycle():
            print(e)
    #print shortest paths
    else:
        for v in range(G.V()):
            if sp.has_path_to(v):
                print("{} to {} ({})  ".format( s, v, sp.dist_to(v)))
                for e in sp.path_to(v):
                    print("{}\t".format(e), end='')
                print()
            else:
                print("{} to {} no path".format(s, v))

if __name__ == '__main__':
    main(sys.argv[1:])

 # *  % python BellmanFordSP.py tinyEWDn.txt 0
 # *  0 to 0 ( 0.00)  
 # *  0 to 1 ( 0.93)  0->2  0.26   2->7  0.34   7->3  0.39   3->6  0.52   6->4 -1.25   4->5  0.35   5->1  0.32
 # *  0 to 2 ( 0.26)  0->2  0.26   
 # *  0 to 3 ( 0.99)  0->2  0.26   2->7  0.34   7->3  0.39   
 # *  0 to 4 ( 0.26)  0->2  0.26   2->7  0.34   7->3  0.39   3->6  0.52   6->4 -1.25   
 # *  0 to 5 ( 0.61)  0->2  0.26   2->7  0.34   7->3  0.39   3->6  0.52   6->4 -1.25   4->5  0.35
 # *  0 to 6 ( 1.51)  0->2  0.26   2->7  0.34   7->3  0.39   3->6  0.52   
 # *  0 to 7 ( 0.60)  0->2  0.26   2->7  0.34   
 # *
 # *  % python BellmanFordSP.py tinyEWDnc.txt 0
 # *  4->5  0.35
 # *  5->4 -0.66
 # *



# Created for BADS 2018
# see README.md for details
# This is python3 
import math
import sys

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.fundamentals.uf import UF
from itu.algs4.sorting.index_min_pq import IndexMinPQ


class PrimMST:
    """
    The PrimMST class represents a data type for computing a
    minimum spanning tree in an edge-weighted graph.
    The edge weights can be positive, zero, or negative and need not
    be distinct. If the graph is not connected, it computes a minimum
    spanning forest, which is the union of minimum spanning trees
    in each connected component. The weight() method returns the 
    weight of a minimum spanning tree and the edges() method
    returns its edges.
    
    This implementation uses Prim's algorithm with an indexed
    binary heap.
    The constructor takes time proportional to E log V
    and extra space not including the graph) proportional to V,
    where V is the number of vertices and E is the number of edges.
    Afterwards, the weight() method takes constant time
    and the edges() method takes time proportional to V.
    """

    FLOATING_POINT_EPSILON = 1E-12

    def __init__(self, G):
        """
        Compute a minimum spanning tree (or forest) of an edge-weighted graph.
        :param G: the edge-weighted graph
        """
        self._edge_to = [None] * G.V()         # self._edge_to[v] = shortest edge from tree vertex to non-tree vertex
        self._dist_to = [0.0] * G.V()       # self._dist_to[v] = weight of shortest such edge
        self._marked = [False] * G.V()      # self._marked[v] = True if v on tree, False otherwise
        self._pq = IndexMinPQ(G.V())
        
        for v in range(G.V()):
            self._dist_to[v] = math.inf

        for v in range(G.V()):              # run from each vertex to find
            if not self._marked[v]:         
                 self._prim(G, v)                  # minimum spanning forest

        # check optimality conditions
        assert  self._check(G)

    # run Prim's algorithm in graph G, starting from vertex s
    def _prim(self, G, s):
        self._dist_to[s] = 0.0
        self._pq.insert(s, self._dist_to[s])
        while not self._pq.is_empty():
            v = self._pq.del_min()
            self._scan(G, v)
    
    def _scan(self, G, v):
        # scan vertex v        
        self._marked[v] = True
        for e in G.adj(v):
            w = e.other(v)
            if self._marked[w]: continue         # v-w is obsolete edge
            if e.weight() < self._dist_to[w]:
                self._dist_to[w] = e.weight()
                self._edge_to[w] = e
                if self._pq.contains(w):    self._pq.decrease_key(w, self._dist_to[w])
                else:                       self._pq.insert(w, self._dist_to[w])


    def edges(self):
        """
        Returns the edges in a minimum spanning tree (or forest).
        :returns: the edges in a minimum spanning tree (or forest) as
                an iterable of edges
        """
        mst = Queue()
        for v in range(len(self._edge_to)):
            e = self._edge_to[v]
            if e is not None:
                mst.enqueue(e)
        
        return mst

    def weight(self):
        """
        Returns the sum of the edge weights in a minimum spanning tree (or forest).
        :returns: the sum of the edge weights in a minimum spanning tree (or forest)
        """
        weight = 0.0
        for e in self.edges():
            weight += e.weight()
        return weight

    def _check(self, G):
        # check optimality conditions (takes time proportional to E V lg* V)
        
        totalWeight = 0.0 # check weight
        for e in self.edges():
            totalWeight += e.weight()
        
        if abs(totalWeight - self.weight()) > PrimMST.FLOATING_POINT_EPSILON:
            error = "Weight of edges does not equal weight(): {} vs. {}\n".format(totalWeight, self.weight())
            print(error, file=sys.stderr)
            return False

        # check that it is acyclic
        uf = UF(G.V())
        for e in self.edges():
            v = e.either()
            w = e.other(v)
            if uf.connected(v, w):
                print("Not a forest", file=sys.stderr)
                return False
            uf.union(v, w)

        # check that it is a spanning forest
        for e in G.edges():
            v = e.either()
            w = e.other(v)
            if not uf.connected(v, w):
                print("Not a spanning forest", file=sys.stderr)
                return False

         # check that it is a minimal spanning forest (cut optimality conditions)
        for e in self.edges():
            # all edges in MST except e
            uf = UF(G.V())
            for f in self.edges():
                x = f.either()
                y = f.other(x)
                if f != e: 
                    uf.union(x, y)            

            # check that e is min weight edge in crossing cut
            for f in G.edges():
                x = f.either()
                y = f.other(x)
                if not uf.connected(x, y):
                    if f.weight() < e.weight():
                        error = "Edge {} violates cut optimality conditions".format(f)
                        print(error, file=sys.stderr)
                        return False
        return True

if __name__ == "__main__":
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.edge_weighted_graph import EdgeWeightedGraph

    In = InStream(sys.argv[1])
    G = EdgeWeightedGraph.from_stream(In)
    mst = PrimMST(G)
    for e in mst.edges():
        stdio.writeln(e)    
    stdio.writef("%.5f\n", mst.weight())    



# Created for BADS 2018
# See README.md for details
# Python 3
import sys

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.stdlib import stdio

# is this really useful??
try:
    q = Queue()
    q.enqueue(1)
except AttributeError:
    print('ERROR - Could not import itu.algs4 queue')
    sys.exit(1)

# Execution:    python lookup_index.py movies.txt "/"
# Dependencies: queue.py stdio.py
# % python lookup_index.py aminoI.csv ","
# Serine
#   TCT
#   TCA
#   TCG
#   AGT
#   AGC
# TCG
#   Serine
#
# % python lookup_index.py movies.txt "/"
# Bacon, Kevin
#   Animal House (1978)
#   Apollo 13 (1995)
#   Beauty Shop (2005)
#   Diner (1982)
#   Few Good Men, A (1992)
#   Flatliners (1990)
#   Footloose (1984)
#   Friday the 13th (1980)
#   ...
# Tin Men (1987)
#   DeBoy, David
#   Blumenfeld, Alan
#   ...

# The LookupIndex class provides a data-driven client for reading in a
# key-value pairs from a file; then, printing the values corresponding to the
# keys found on standard input. Keys are strings; values are lists of strings.
# The separating delimiter is taken as a command-line argument. This client
# is sometimes known as an inverted index.
if __name__ == '__main__':
    args = sys.argv[1:]
    filename  = args[0]
    separator = args[1]
    file = open(filename, 'r')
    st = {}
    ts = {}

    line = file.readline()
    while line:
        fields = line.split(separator)
        key = fields[0]
        for i in range(1, len(fields)):
            val = fields[i]
            if not key in st:       
                st[key] = Queue()
            if not val in ts:       
                ts[val] = Queue()
            st.get(key).enqueue(val)
            ts.get(val).enqueue(key)
        line = file.readline()
    print("Done indexing")

    #read queries from standard input, one per line
    while not stdio.isEmpty():
        query = stdio.readLine()
        if query in st:
            for vals in st.get(query):
                print("  " + vals)
        if query in ts:
            for keys in ts.get(query):
                print("  " + keys)

    file.close()



# created for BADS 2018
# See README.md for details
# Python 3

import sys

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.stdlib import stdio

try:
    q = Queue()
    q.enqueue(1)
except AttributeError:
    print('ERROR - Could not import itu.algs4 queue')
    sys.exit(1)


"""
 *  The TrieST class represents an symbol table of key-value
 *  pairs, with string keys and generic values.
 *  It supports the usual put, get, contains,
 *  delete, size, and is-empty methods.
 *  It also provides character-based methods for finding the string
 *  in the symbol table that is the longest prefix of a given prefix,
 *  finding all strings in the symbol table that start with a given prefix,
 *  and finding all strings in the symbol table that match a given pattern.
 *  A symbol table implements the associative array abstraction:
 *  when associating a value with a key that is already in the symbol table,
 *  the convention is to replace the old value with the new value.
 *  This class uses the convention that
 *  values cannot be None, setting the
 *  value associated with a key to None is equivalent to deleting the key
 *  from the symbol table.
 *  This implementation uses a 256-way trie.
 *  The put, contains, delete, and
 *  longest prefix operations take time proportional to the length
 *  of the key (in the worst case). Construction takes constant time.
 *  The size, and is-empty operations take constant time.
 """


class TrieST(object):
    R = 256     # extended ASCII

    # R-way trie node
    class Node(object):
        R = 256
        def __init__(self):
            self.val = None
            self.next = [None] * self.R  # array of nodes of length R 

    def __init__(self):
        self._root = self.Node()    # root of trie
        self._n = 0                 # number of keys in trie


    # Returns the value associated with the given key.
    # @param key: the key
    # @return: the value associated with the given key if the key is in the symbol table
    #     and None if the key is not in the symbol table
    # @raises TypeError if key is None
    
    def get(self, key):
        x = self._get(self._root, key, 0)
        return None if x is None else x.val

    # Does this symbol table contain the given key?
    # @param key: the key
    # @returns True if this symbol table contains key and
    #     False otherwise
    # @raises TypeError if key is None

    def contains(self, key):
        return self.get(key) is not None
    
    def _get(self, x, key, d):
        if x is None:
            return None
        if d == len(key):
            return x
        c = key[d]
        return self._get(x.next[ord(c)], key, d+1) 

    # Inserts the key-value pair into the symbol table, overwriting the old value
    # with the new value if the key is already in the symbol table.
    # If the value is None, this effectively deletes the key from the symbol table.
    # @param key: the key
    # @param val: the value
    # @raises TypeError if key is None
    
    def put(self, key, val):
        if val is None:
            self.delete(key)
        else:
            self._root = self._put(self._root, key, val, 0)

    def _put(self, x, key, val, d):
        if x is None:
            x = self.Node()
        if d == len(key):
            if x.val is None:
                self._n +=1
            x.val = val
            return x
        c = key[d]
        x.next[ord(c)] = self._put(x.next[ord(c)], key, val, d+1)
        return x
    
    # @return the number of key-value pairs in this symbol table
    def size(self):
        return self._n

    def __len__(self):
        return self.size()

    # Is this symbol table empty?
    # @return True if this symbol table is empty and False otherwise
    def is_empty(self):
        return self.size() == 0
    
    
    # Returns all keys in the symbol table as an iterable object.
    # To iterate over all of the keys in the symbol table named st,
    # use the foreach notation: for key in st.keys().
    # @return all keys in the symbol table as an iterable object
    
    def keys(self):
        return self.keys_with_prefix('')

    # Returns all of the keys in the set that start with prefix.
    # @param prefix: the prefix
    # @return all of the keys in the set that start with prefix,
    #     as an iterable
    
    def keys_with_prefix(self, prefix):
        results = Queue()
        x = self._get(self._root, prefix, 0)
        self._collect(x, prefix, results)
        return results
    
    def _collect(self, x, prefix, results):
        if x is None:
            return
        if x.val is not None:
            results.enqueue(prefix)
        for c in range(0, self.R):
            self._collect(x.next[c], prefix + chr(c), results)

    # Returns all of the keys in the symbol table that match pattern,
    # where . symbol is treated as a wildcard character.
    # @param pattern the pattern
    # @return all of the keys in the symbol table that match pattern,
    #     as an iterable, where . is treated as a wildcard character.
    
    def keys_that_match(self, pattern):
        results = Queue()
        self._collect_match(self._root, '', pattern, results) 
        return results
    
    def _collect_match(self, x, prefix, pattern, results):
        if x is None:
            return None
        d = len(prefix)
        if d == len(pattern) and x.val is not None:
            results.enqueue(prefix) 
        if d >= len(pattern):
            return
        c = pattern[d]
        if c == '.':
            for c in range(0, self.R): 
                self._collect_match(x.next[c], prefix + chr(c), pattern, results) 
        else:
            self._collect_match(x.next[ord(c)], prefix + c, pattern, results)

    
    # Returns the string in the symbol table that is the longest prefix of query,
    # or None, if no such string.
    # @param query the query string
    # @return the string in the symbol table that is the longest prefix of query,
    #  or None if no such string
    # @raises TypeError if query is None
    
    def longest_prefix_of(self, query):
        length = self._longest_prefix_of(self._root, query, 0, -1)
        if length == -1:
            return None
        else:
            return query[:length]

    # returns the length of the longest string key in the subtrie
    # rooted at x that is a prefix of the query string,
    # assuming the first d character match and we have already
    # found a prefix match of given length (-1 if no such match)
    
    def _longest_prefix_of(self, x, query, d, length):
        if x is None:
            return length
        if x.val is not None:
            length = d
        if d == len(query):
            return length
        c = query[d]
        return self._longest_prefix_of(x.next[ord(c)], query, d+1, length) 
  
    # Removes the key from the set if the key is present.
    # @param key the key
    # @raises TypeError if key is None
    
    def delete(self, key):
        self._root = self._delete(self._root, key, 0)


    def _delete(self, x, key, d):
        if x is None:
            return None
        if d == len(key):
            if x.val is not None:
                self._n += -1
            x.val = None
        else:
            c = key[d]
            x.next[ord(c)] = self._delete(x.next[ord(c)], key, d+1) 
        
        # remove subtrie rooted at x if it is completely empty
        if x.val is not None:
            return x
        for c in range(0, self.R):
            if x.next[c] is not None:
                return x
        return None

def test():
    st = TrieST()
    st.put("abc", 0)
    keys = [k for k in st.keys()]
    assert keys == ["abc"]
    assert st.get("abc") == 0
    st.put("a", 1)
    st.put("b", 2)
    st.put("c", 3)
    st.delete("abc")
    assert "abc" not in [k for k in st.keys()]
    assert st.get("a") == 1
    assert st.get("b") == 2
    assert st.get("c") == 3
    st.put("hello", 10)
    assert st.contains("hello")
    assert st.contains("not there") is False
    st.put("he", 20)
    assert st.longest_prefix_of("hell") == "he"
    st.put("jello", 30)
    q = st.keys_that_match(".e.l.")
    assert q.size() == 2 and "jello" in q and "hello" in q
    print("tests passed.")

if __name__ == '__main__':
    test()
    st = TrieST()
    i = 0
    print("Insert keys (Ctrl-D to stop):")
    while not stdio.isEmpty(): 
        key = stdio.readString()
        st.put(key, i)
        i += 1
    # print results
    if st.size() < 100:
        print('keys(""):')
        for key in st.keys():
            print('{} {}'.format(key, st.get(key)))
        print()
    print('longest_prefix_of("shellsort"):')
    print(st.longest_prefix_of('shellsort'))
    print()
    print('longest_prefix_of("quicksort")')
    print(st.longest_prefix_of('quicksort'))
    print()
    print('keys_with_prefix("shor")')
    for s in st.keys_with_prefix('shor'):
        print(s)
    print()
    print('keys_that_match("he.l.")')
    for s in st.keys_that_match('he.l.'):
        print()



# Created for BADS 2018
# See README.md for details
# this one is not inspired by algorithms from the book, but useful for running java and python in parallel
# Python 3

def java_string_hash(key):
    """ If key is a string, compute its java .hash() code. Taken from http://garage.pimentech.net/libcommonPython_src_python_libcommon_javastringhashcode/ """
    h = 0
    for c in key:
        h = (31 * h + ord(c)) & 0xFFFFFFFF
    return ((h + 0x80000000) & 0xFFFFFFFF) - 0x80000000

def trailing_zeros(i):
    zeros = 0
    while i & 1 == 0 and zeros < 32:
        zeros += 1
        i = i >> 1
    return zeros



import sys
from typing import Generic, List, TypeVar

from itu.algs4.stdlib import stdio

T = TypeVar('T')

# Created for BADS 2018
# See README.md for details
# This is python3

"""
The binary_search module provides a method for binary
searching for an item in a sorted array.
The index_of operation takes logarithmic time in the worst case.
"""


def index_of(a: List[T], key: T):
    """
    Returns the index of the specified key in the specified array.
    :param a: the array of items, must be sorted in ascending order
    :param key: the search key
    :return: index of key in array if present -1 otherwise
    """
    lo = 0
    hi = len(a) - 1
    while hi >= lo:
        mid = lo + (hi - lo)//2
        if a[mid] < key:
            lo = mid + 1
        elif a[mid] > key:
            hi = mid - 1
        else:
            return mid
    return -1


def main():
    """
    Reads strings from first input file and sorts them
    Reads strings from second input file and prints every string not in first input file
    """
    if len(sys.argv) == 3:
        sys.stdin = open(sys.argv[1])
        arr = stdio.readAllStrings()
        arr.sort()
        sys.stdin = open(sys.argv[2])
        while not stdio.isEmpty():
            key = stdio.readString()
            if index_of(arr, key) == -1:
                print(key)


if __name__ == '__main__':
    main()



# Created for BADS 2018
# See README.md for details
# Python 3

import sys

from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.directed_edge import DirectedEdge
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.graphs.topological import Topological
from itu.algs4.stdlib import instream


"""
This module implements a class for solving the single-source Longest
paths problem in edge-weighted directed acyclic graphs (DAGs), described in 
Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne. 

For more information, see chapter 4.2 of the book.
"""

class AcyclicLp:
    """
    The AcyclicLP class represents a data type for solving the single-source longest 
    paths problem in edge-weighted directed acyclic graphs (DAGs). The edge weights 
    can be positive, negative, or zero.
    
    This implementation uses a topological-sort based algorithm. The constructor takes 
    time proportional to V + E, where V is the number of vertices and E is the number of edges. 
    Each call to distTo(int) and hasPathTo(int) takes constant time; each call to pathTo(int) 
    takes time proportional to the number of edges in the shortest path returned.

    For additional documentation, see Section 4.4 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
    """
    
    def __init__(self, edge_weighted_digraph, s):
        """
        Computes a longest paths tree from s to every other vertex in the directed acyclic graph G.
        
        :param edge_weighted_digraph: the acyclic digraph
        :param s: the source vertex
        """
        graph = edge_weighted_digraph
        self._dist_to = [float("-inf")]*graph.V()
        self._edge_to = [None]*graph.V()
        
        self._validate_vertex(s)
        
        self._dist_to[s] = 0.
        
        # relax vertices in topological order
        topological = Topological(graph)
        if not topological.has_order():
            print(graph)
            raise ValueError('Digraph is not acyclic.')
        
        for v in topological.order():
            for edge in graph.adj(v):
                self._relax(edge)
                
    def dist_to(self, v):
        """
        Returns the length of a longest path from the source vertex s to vertex v.
        
        :param v: the destination vertex
        
        :returns: the length of a longest path from the source vertex s to vertex v; 
                  negative infinity if no such path exists
        """
        self._validate_vertex(v)
        return self._dist_to[v]
    
    def has_path_to(self, v):
        """
        Is there a path from the source vertex s to vertex v?
        """
        return self.dist_to(v) > float("-inf")
    
    def path_to(self, v):
        """
        Returns a longest path from the source vertex s to vertex v.
        
        :param: the destination vertex
        :returns: a longest path from the source vertex s to vertex v as an iterable of edges, and None if no such path
        """
        self._validate_vertex(v)
        if not self.has_path_to(v):
            return None
        path = Stack()
        edge = self._edge_to[v]
        while not edge is None:
            path.push(edge)
            edge = self._edge_to[edge.from_vertex()]
        return path
        
                
    # relax edge e, but update if you find a *longer* path 
    def _relax(self, edge):
        v, w = edge.from_vertex(), edge.to_vertex()
        if self._dist_to[w] < self._dist_to[v] + edge.weight():
            self._dist_to[w] = self._dist_to[v] + edge.weight()
            self._edge_to[w] = edge
        
    # throw an IllegalArgumentException unless 0 <= v < V
    def _validate_vertex(self, v):
        V = len(self._dist_to)
        if not (0 <= v < V):
            raise ValueError('vertex {} is not between 0 and {}'.format(v, V-1))



if __name__ == '__main__':
    # Create stream from file or the standard input,
    # depending on whether a file name was passed.
    stream = sys.argv[1] if len(sys.argv) > 1 else None
    d = EdgeWeightedDigraph.from_stream(instream.InStream(stream))
    a = AcyclicLp(d, 3)
    
    # print longest paths to all other vertices
    for i in range(d.V()):
        print('{} to {}'.format(i, a.path_to(i)))



# Created for BADS 2018
# see README.md for details
# This is python3 

import math

from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.topological import Topological


class AcyclicSP:
    """
    The AcyclicSP class represents a data type for solving the
    single-source shortest paths problem in edge-weighted directed acyclic
    graphs (DAGs). The edge weights can be positive, negative, or zero.

    This implementation uses a topological-sort based algorithm.
    The constructor takes time proportional to V + E,
    where V is the number of vertices and E is the number of edges.
    Each call to distTo(int) and has_path_to(int) takes constant time
    each call to pathTo(int) takes time proportional to the number of
    edges in the shortest path returned.
    """

    def __init__(self, G, s):
        """
        Computes a shortest paths tree from s to every other vertex in
        the directed acyclic graph G.
        :param G: the acyclic digraph
        :param s: the source vertex
        :raises ValueError: if the digraph is not acyclic
        :raises ValueError: unless 0 <= s < V
        """
        self._dist_to = [0] * G.V()         # _dist_to[v] = distance  of shortest s->v path
        self._edge_to = [None] * G.V()      # _edge_to[v] = last edge on shortest s->v path

        self._validate_vertex(s)

        for v in range(G.V()):
            self._dist_to[v] = math.inf
        self._dist_to[s] = 0.0

        # visit vertices in toplogical order
        topological = Topological(G)
        if not topological.has_order():
            raise ValueError("Digraph is not acyclic.")

        for v in topological.order():
            for e in G.adj(v):
                self._relax(e)
                
    def _relax(self, e):
        v = e.from_vertex()
        w = e.to_vertex()

        if self._dist_to[w] > self._dist_to[v] + e.weight():
            self._dist_to[w] = self._dist_to[v] + e.weight()
            self._edge_to[w] = e
        
    def dist_to(self, v):
        """
        Returns the length of a shortest path from the source vertex s to vertex v.
        :param v: the destination vertex
        :returns: the length of a shortest path from the source vertex s to vertex v
                math.inf if no such path
        :raises ValueError: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._dist_to[v]
    
    
    def has_path_to(self, v):
        """
        Is there a path from the source vertex s to vertex v?
        :param v: the destination vertex
        :returns: true if there is a path from the source vertex
                s to vertex v, and false otherwise
        :raises ValueError: unless 0 <= v < V
        """
        self._validate_vertex(v)
        return self._dist_to[v] < math.inf
    
    def path_to(self, v):
        """
        Returns a shortest path from the source vertex s to vertex v.
        :param v: the destination vertex
        :returns: a shortest path from the source vertex s to vertex v
                as an iterable of edges, and None if no such path
        :raises ValueError: unless 0 <= v < V
        """
        self._validate_vertex(v)
        if not self.has_path_to(v): return None
        path =  Stack()
        e = self._edge_to[v]
        while e is not None:
            path.push(e)
            e = self._edge_to[e.from_vertex()]        
        return path
    
    def _validate_vertex(self, v):
        # raise an ValueError unless 0 <= v < V
        V = len(self._dist_to)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))

if __name__ == "__main__":
    import sys
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph

    In = InStream(sys.argv[1])
    s = int(sys.argv[2])
    G = EdgeWeightedDigraph.from_stream(In)

    # find shortest path from s to each other vertex in DAG
    sp = AcyclicSP(G, s)
    for v in range(G.V()):
        if sp.has_path_to(v):
            stdio.writef("%d to %d (%.2f)  ", s, v, sp.dist_to(v))
            for e in sp.path_to(v):
                stdio.writef("%s\t", e.__repr__())
            stdio.writeln()
        else:
            stdio.writef("%d to %d no path\n", s, v)



# Created for BADS 2018
# see README.md for details
# This is python3 

from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.graph import Graph


class Bipartite:
    """
    The Bipartite class represents a data type for 
    determining whether an undirected graph is bipartite or whether
    it has an odd-length cycle.
    The isBipartite operation determines whether the graph is
    bipartite. If so, the color operation determines a
    bipartition if not, the oddCycle operation determines a
    cycle with an odd number of edges.
    
    This implementation uses depth-first search.
    The constructor takes time proportional to V + E
    (in the worst case),
    where V is the number of vertices and E is the number of edges.
    Afterwards, the isBipartite and color operations
    take constant time the oddCycle operation takes time proportional
    to the length of the cycle.
    See BipartiteX for a nonrecursive version that uses breadth-first
    search.
    """

    class UnsupportedOperationException(Exception):
        pass

    def __init__(self, G):
        """
        Determines whether an undirected graph is bipartite and finds either a
        bipartition or an odd-length cycle.

        :param G: the graph
        """
        self._is_bipartite = True       # is the graph bipartite?
        self._color = [False] * G.V()   # color[v] gives vertices on one side of bipartition
        self._marked = [False] * G.V()  # marked[v] = True if v has been visited in DFS
        self._edge_to = [0] * G.V()      # edgeTo[v] = last edge on path to v
        self._cycle = None              # odd-length cycle
        
        for v in range(G.V()):
            if not self._marked[v]:
                self._dfs(G, v)

        assert self._check(G)
    
    def _dfs(self, G, v):
        self._marked[v] = True

        for w in G.adj(v):
            # short circuit if odd-length cycle found
            if self._cycle is not None: return

            # found uncolored vertex, so recur
            if not self._marked[w]:
                self._edge_to[w] = v
                self._color[w] = not self._color[v]
                self._dfs(G, w)             

            # if v-w create an odd-length cycle, find it
            elif self._color[w] == self._color[v]:
                self._is_bipartite = False
                self._cycle = Stack()
                self._cycle.push(w)  # don't need this unless you want to include start vertex twice
                x = v
                while x != w:
                    self._cycle.push(x)
                    x = self._edge_to[x]

                self._cycle.push(w)
            
    def is_bipartite(self):
        """
        Returns True if the graph is bipartite.
        
        :returns: True if the graph is bipartite False otherwise
        """
        return self._is_bipartite

    def color(self, v):
        """
        Returns the side of the bipartite that vertex v is on.

        :param v: the vertex
        :returns: the side of the bipartition that vertex v is on two vertices
                are in the same side of the bipartition if and only if they have the
                same color
        :raises IllegalArgumentException: unless 0 <= v < V 
        :raises UnsupportedOperationException: if this method is called when the graph
                is not bipartite
        """
        self._validateVertex(v)
        if not self._is_bipartite:
            raise Bipartite.UnsupportedOperationException("graph is not bipartite")
        return self._color[v]

    def odd_cycle(self):
        """
        Returns an odd-length cycle if the graph is not bipartite, and
        None otherwise.
        
        :returns: an odd-length cycle if the graph is not bipartite
                (and hence has an odd-length cycle), and None otherwise
        """
        return self._cycle

    def _check(self, G):
        # graph is bipartite
        if self._is_bipartite: 
            for v in range(G.V()):
                for w in G.adj(v):
                    if self._color[v] == self._color[w]:
                        error = "edge {}-{} with {} and {} in same side of bipartition\n".format(v, w, v, w)
                        print(error, file=sys.stderr)
                        return False
        # graph has an odd-length cycle
        else:
            # verify cycle
            first = -1
            last = -1
            for v in self.odd_cycle():
                if first == -1:
                    first = v
                last = v
            
            if first != last:
                error = "cycle begins with {} and ends with {}\n".format(first, last)
                print(error, file=sys.stderr)
                return False
        return True
    
    def _validateVertex(self, v):
        # raise an ValueError unless 0 <= v < V    
        V = len(self._marked)
        if v < 0 or v >= V:
            raise ValueError("vertex {} is not between 0 and {}".format(v, V-1))


if __name__ == "__main__":
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    import sys
    
    In = InStream(sys.argv[1])
    G = Graph.from_stream(In)
    stdio.writeln(G)

    b = Bipartite(G)
    if b.is_bipartite():
        stdio.writeln("Graph is bipartite")
        for v in range(G.V()):
            stdio.writef("%i: %i\n", v, b.color(v))
    else:
        stdio.writeln("Graph has an odd-length cycle: ")
        for x in b.odd_cycle():
            stdio.writef("%i ", x)
        stdio.writeln()



# Created for BADS 2018
# See README.md for details
# This is python3 
import sys


class LinearProbingHashST:
	"""
	The LinearProbingHashST class represents a symbol table of dynamic
	key-value pairs.
	It supports the usual put, get, contains, delete, size, 
	and is-empty methods.
	It also provides a key_list method for iterating over all of the keys.
	A symbol table implements the associative array abstraction:
	when associating a value with a key that is already in the symbol table,
	the convention is to replace the old value with the new value.
	Unlike the Map-class in Java, this class uses the convention that
	values cannot be null/None. Setting the
	value associated with a key to None is equivalent to deleting the key
	from the symbol table.

	This implementation uses a linear probing hash table. It requires that
	the key type overrides the __eq__ and __hash__ methods.
	The expected time per put, contains, or remove
	operation is constant, subject to the uniform hashing assumption.
	The size, and is-empty operations take constant time.
	Construction takes constant time.
	"""
	def __init__(self, capacity = 4):
		"""
		Initializes an empty symbol table with the specified initial capacity.
		If no capacity is specified, it defaults to 4.

		:param capacity: the initial capacity
		"""
		self.m = capacity
		self.n = 0
		self.keys = [None for i in range(0,self.m)]
		self.values = [None for i in range(0,self.m)]

	def size(self):
		"""
		Returns the number of key-value pairs in this symbol table.

		:returns: the number of key-value pairs in this symbol table.
		"""
		return self.n

	def __len__(self):
                return self.size()

	def is_empty(self):
		"""
		Returns True if this symbol table is empty.

		:returns: True if this symbol table is empty;
					False otherwise
		"""
		return self.n == 0

	def contains(self, key):
		"""
		Returns True if this symbol table contains the specified key.

		:param key: the key
		:returns: True if this symbol table contains the key;
					False otherwize
		:raises ValueError: if key is None
		"""
		if(key is None):
			raise ValueError("argument to contains() is None")
		return self.get(key) is not None

	def _hash(self, key):
		#Hash value between 0 and M-1
		return (hash(key) & 0x7fffffff) % self.m

	def _resize(self, capacity):
		# Resizes the hash table to the given capacity by re-hashing all of the keys
		temp = LinearProbingHashST(capacity)
		for i in range(0,self.m):
			if(self.keys[i] is not None):
				temp.put(self.keys[i], self.values[i])
		self.keys = temp.keys
		self.values = temp.values
		self.m = temp.m

	def put(self, key, value):
		"""
		Inserts the specified key-value paur into the symbol table, overwriting the old
		value with the new value if the symbol table already contains the specified key.
		Deletes the specified key (and its associated value) from this symbol table
		if the specified value is None.

		:param key: the key
		:param value: the value
		:raises ValueError: if key is None.
		"""
		if(key is None):
			raise ValueError("argument to put() is None")
		if(value is None):
			self.delete(key)
			return
		# Double table size if 50% full
		if (self.n >= self.m//2):
			self._resize(2*self.m)

		i = self._hash(key)
		while(self.keys[i] is not None):
			if(self.keys[i] == key):
				self.values[i] = value
				return
			i = (i+1) % self.m
		self.keys[i] = key
		self.values[i] = value
		self.n += 1

	def get(self, key):
		"""
		Returns the value associated with the specified key.

		:param key: the key
		:returns: the value associated with the key in the symbol table;
					None if no such value
		:raises ValueError: if key is None
		"""
		if(key is None):
			raise ValueError("argument to get() is None")
		i = self._hash(key)
		while(self.keys[i] is not None):
			if(self.keys[i] == key):
				return self.values[i]
			i = (i+1) % self.m
		return None

	def delete(self, key):
		"""
		Removes the specified key and its associated value from this symbol table
		(if the key is in this symbol table).

		:param key: the key
		:raises ValueError: if key is None
		"""
		if(key is None):
			raise ValueError("argument to delete() is None")
		if(not self.contains(key)):
			return
		# Find position i of key
		i = self._hash(key)
		while(not key == self.keys[i]):
			i = (i+1) % self.m
		# Delete key and associated value
		self.keys[i] = None
		self.values[i] = None
		# Rehash all keys in same cluster
		i = (i+1) % self.m
		while(self.keys[i] is not None):
			# Delete keys[i] and values[i] and reinsert
			keyToRehash = self.keys[i]
			valueToReash = self.values[i]
			self.keys[i] = None
			self.values[i] = None
			self.n -= 1
			self.put(keyToRehash, valueToReash)
			i = (i+1) % self.m
		self.n -= 1

		# Halves table size if it's less than 12.5% full
		if(self.n > 0 and self.n <= self.m/8):
			self._resize(self.m//2)

		assert self._check()

	def key_list(self):
		"""
		Returns the keys in the symbol table as an iterable
		:returns: A list containing all keys
		"""
		keys = []
		for i in range(0,self.m):
			if(self.keys[i] is not None):
				keys.append(self.keys[i])
		return keys

	def _check(self):
		# Integrity check - don't check after each put() because
		# integrity is not maintained during delete()
		if(self.m < 2*self.n):
			print("Hash table size m = {}; List size n = {}".format(self.m, self.n))
			return False
		for i in range(0,self.m):
			if(self.keys[i] is None):
				continue
			elif(self.get(self.keys[i]) != self.values[i]):
				print("get[{}] = {}; values[i] = {}".format(self.keys[i], self.get(self.keys[i]), self.values[i]))
				return False
		return True

def main():
	"""
	Unit tests the LinearProbingHashST data type.
	"""
	st = LinearProbingHashST()
	i = 1
	for key in sys.argv[1:]:
		st.put(key, i)
		i += 1
	for key in st.key_list():
		print("{} {}".format(key, st.get(key)))
if __name__ == "__main__":
    main()



# Created for BADS 2018
# See README.md for details
# Python 3

import sys

import itu.algs4.graphs.directed_edge
from itu.algs4.graphs.acyclic_lp import AcyclicLp
from itu.algs4.graphs.directed_edge import DirectedEdge
from itu.algs4.graphs.edge_weighted_digraph import EdgeWeightedDigraph
from itu.algs4.stdlib import instream

"""
The cpm module is an example of using graphs to solve the parallel precedence-constrained 
job scheduling problem via the critical path method. It reduces the problem to the longest-paths 
problem in edge-weighted DAGs. It builds an edge-weighted digraph (which must be a DAG) from the 
job-scheduling problem specification, finds the longest-paths tree, and computes the longest-paths 
lengths (which are precisely the start times for each job).

This implementation uses AcyclicLP to find a longest path in a DAG. The running time is proportional 
to V + E, where V is the number of jobs and E is the number of precedence constraints.

For additional documentation, see Section 4.4 of Algorithms, 4th Edition by Robert Sedgewick and Kevin Wayne.
"""



# Try this with the jobsPC.txt data file
if __name__ == '__main__':
    # Create stream from file or the standard input,
    # depending on whether a file name was passed.    
    file = sys.argv[1] if len(sys.argv) > 1 else None
    stream = instream.InStream(file)
    
    # Number of jobs
    N = stream.readInt()

    # Source and sink
    source, sink = 2*N, 2*N+1
    
    # Construct the network
    G = EdgeWeightedDigraph(2*N+2)
    for i in range(N):
        duration = stream.readFloat()
        G.add_edge(DirectedEdge(i, i+N, duration))
        G.add_edge(DirectedEdge(source, i, 0.0))
        G.add_edge(DirectedEdge(i+N, sink, 0.0))
        
        # Precedence constraints
        m = stream.readInt()
        for _ in range (m):
            successor = stream.readInt()
            G.add_edge(DirectedEdge(i+N, successor, 0.0))
    
    # Compute longest path
    lp = AcyclicLp(G, source)
    
    # Print results
    print('Start times:')
    for i in range(N):
        print('{:4d}: {:5.1f}'.format(i, lp.dist_to(i)))
    print('Finish time: {:5.1f}'.format(lp.dist_to(i)))



from abc import abstractmethod
from typing import Generic, Optional, TypeVar

from typing_extensions import Protocol

from ..errors.errors import IllegalArgumentException, NoSuchElementException
from ..fundamentals.queue import Queue

# Created for BADS 2018
# See README.md for details
# Python 3

# Typing ---


Key = TypeVar('Key', bound = "Comparable")
Val = TypeVar('Val')

class Comparable(Protocol):
    @abstractmethod
    def __lt__(self: Key, other: Key) -> bool:
        pass

# ---
class Node(Generic[Key, Val]):
    """
    RedBlackBST helper node data type.
    """
    def __init__(self, key: Key, val: Val, color: bool, size: int):
        """
        Initializes a new node.
        :param key: the key of the node
        :param val: the value of the node
        :param size: the subtree count
        """
        self.key: Key = key
        self.val: Val = val
        self.left:  Optional[Node[Key, Val]] = None
        self.right: Optional[Node[Key, Val]] = None
        self.size:  int = size
        self.color: bool = color

class RedBlackBST(Generic[Key, Val]):
    """
    The RedBlackBST class represents an ordered symbol table of generic
    key-value pairs.
    It supports the usual put, get, contains,
    delete, size, and is-empty methods.
    It also provides ordered methods for finding the minimum,
    maximum, floor, and ceiling.
    It also provides a keys method for iterating over all the keys.
    A symbol table implements the associative array abstraction:
    when associating a value with a key that is already in the symbol table,
    the convention is to replace the old value with the new value.
    This class uses the convention that values cannot be None-setting the
    value associated with a key to None is equivalent to deleting the key from the symbol table.
    This implementation uses a left-leaning red-black BST. It requires that
    the keys are all of the same type and that they can be compared.
    The put, contains, remove, minimum, maximum, ceiling, and floor operations each take
    logarithmic time in the worst case, if the tree becomes unbalanced.
    The size, and is-empty operations take constant time.
    Construction takes constant time.
    """

    RED = True
    BLACK = False


    def __init__(self) -> None:
        """
        Initializes an empty symbol table.
        """
        self._root : Optional[Node[Key, Val]] = None

    def put(self, key: Key, val: Val) -> None:
        """
        Inserts the specified key-value pair into the symbol table, overwriting the old
        value with the new value if the symbol table already contains the specified key.
        Deletes the specified key (and its associated value) from this symbol table
        if the specified value is None.
        :param key: the key
        :param val: the value
        :raises IllegalArgumentException: if key is None
        """
        # Can never happen if type checked:
        if key is None:
            raise IllegalArgumentException("first argument to put() is None")
        if val is None:
            self.delete(key)
            return

        self._root = self._put(self._root, key, val)
        self._root.color = RedBlackBST.BLACK

    def _put(self, h: Optional[Node[Key, Val]], key: Key, val: Val) -> Node[Key, Val]:
        """
        Inserts the key-value pair in the subtree rooted at h.
        :param h: root of currently inspected subtree
        :param key: the key
        :param val: the value
        """
        if h is None:
            return Node(key, val, self.RED, 1)
        if key < h.key:
            h.left = self._put(h.left, key, val)
        elif key > h.key:
            h.right = self._put(h.right, key, val)
        else:
            h.val = val

        assert h is not None
        if self._is_red(h.right) and not self._is_red(h.left):
            h = self._rotate_left(h)
        assert h is not None
        if self._is_red(h.left):
            assert h.left is not None # bc h.left is red
            if self._is_red(h.left.left): 
                h = self._rotate_right(h)
        assert h is not None
        if self._is_red(h.left) and self._is_red(h.right):
            self._flip_colors(h)

        h.size = self._size(h.left) + self._size(h.right) + 1

        return h

    def get(self, key: Key) -> Optional[Val]:
        """
        Returns the value associated with the given key.
        :param key: the key
        :return: the value associated with the given key if the key is in the symbol table
        and None if the key is not in the symbol table
        :raises IllegalArgumentException: if key is None
        """
        # This can never happen if type checked:
        if key is None:
            raise IllegalArgumentException("argument to get() is None")

        return self._get(self._root, key)

    def _get(self, x: Optional[Node[Key, Val]], key: Key) -> Optional[Val]:
        """
        Returns value with the given key in subtree rooted at x.
        None if no such key.
        :param x: root of currently inspected subtree.
        :param key: the key
        :return: value associated with given key. None if no such key
        """
        while x is not None:
            if key < x.key:
                x = x.left
            elif key > x.key:
                x = x.right
            else:
                return x.val
        return None

    def delete_min(self) -> None:
        """
        Removes the smallest key and associated value from the symbol table.
        :raises NoSuchElementException: if the symbol table is empty
        """
        if self.is_empty():
            raise NoSuchElementException("RedBlackBST underflow")
        assert self._root is not None
        if not self._is_red(self._root.left) and not self._is_red(self._root.right):
            self._root.color = self.RED
        self._root = self._delete_min(self._root)
        if not self.is_empty():
            assert self._root is not None
            self._root.color = RedBlackBST.BLACK

    def _delete_min(self, h: Node[Key, Val]) -> Optional[Node[Key,Val]]:
        """
        Deletes the node with the minimum key rooted at h.
        """
        if h.left is None:
            return None
        if not self._is_red(h.left) and not self._is_red(h.left.left):
            h = self._move_red_left(h)
        assert h.left is not None # because _move_red_left moved something to h.left
        h.left = self._delete_min(h.left)
        return self._balance(h)

    def delete_max(self) -> None:
        """
        Removes the largest key and associated value from the symbol table.
        :raises NoSuchElementException: if the symbol table is empty
        """
        if self.is_empty():
            raise NoSuchElementException("RedBlackBST underflow")
        assert self._root is not None
        if not self._is_red(self._root.left) and not self._is_red(self._root.right):
            self._root.color = self.RED
        self._root = self._delete_max(self._root)
        if not self.is_empty():
            assert self._root is not None
            self._root.color = RedBlackBST.BLACK

    def _delete_max(self, h: Node[Key, Val]) -> Optional[Node[Key,Val]]:
        """
        Deletes the key-value pair with the maximum key rooted at h
        """
        if self._is_red(h.left):
            h = self._rotate_right(h)
        if h.right is None:
            return None
        if not self._is_red(h.right) and not self._is_red(h.right.left):
            h = self._move_red_right(h)
        assert h.right is not None # because _move_red_right moved something to h.right
        h.right = self._delete_max(h.right)
        return self._balance(h)

    def delete(self, key: Key) -> None:
        """
        Removes the specified key and its associated value from this symbol table
        (if the key is in this symbol table).
        :param key: the key
        :raises IllegalArgumentException: if key is None
        """
        # Cannot happen if type checked:
        if key is None:
            raise IllegalArgumentException("argument to delete() is None")
        if not self.contains(key):
            return
        assert self._root is not None
        if not self._is_red(self._root.left) and not self._is_red(self._root.right):
            self._root.color = self.RED
        self._root = self._delete(self._root, key)
        if not self.is_empty():
            assert self._root is not None
            self._root.color = RedBlackBST.BLACK

    def _delete(self, h: Node[Key, Val], key: Key) -> Optional[Node[Key, Val]]:
        """
        Deletes the key-value pair with the given key rooted at h.
        """
        if key < h.key:
            # we assert (from delete) that key exists in h's subtree, so it must exists
            # in the left subtree, so h.left is not None
            assert h.left is not None
            if not self._is_red(h.left) and not self._is_red(h.left.left):
                h = self._move_red_left(h)
            assert h.left is not None # _move_red_left does what it should
            h.left = self._delete(h.left, key)
        else:
            if self._is_red(h.left):
                h = self._rotate_right(h)
            if key == h.key and h.right is None:
                return None
            assert h.right is not None # bc. key must be in the right subtree 
            if not self._is_red(h.right) and not self._is_red(h.right.left):
                h = self._move_red_right(h)
            assert h.right is not None
            if key == h.key:
                x = self._min(h.right)
                h.key = x.key
                h.val = x.val
                h.right = self._delete_min(h.right)
            else:
                h.right = self._delete(h.right, key)
        return self._balance(h)

    def size(self) -> int:
        """
        Return the number of key-value pairs in this symbol table.
        :return: the number of key-value pairs in this symbol table
        """
        return self._size(self._root)

    def __len__(self) -> int:
        return self.size()

    def _size(self, x: Optional[Node[Key, Val]]) -> int:
        """
        Number of nodes in subtree rooted at x. 0 if x is None
        :param x: root node of subtree
        :return: number of nodes in subtree
        """
        if x is None:
            return 0
        return x.size

    def contains(self, key: Key) -> bool:
        """
        Does this symbol table contain the given key?
        :param key: the key
        :return: True if this symbol table contains key and False otherwise
        """
        return self.get(key) is not None

    def is_empty(self) -> bool:
        """
        Is this symbol table empty?
        :return: True if this symbol table is empty and False otherwise
        """
        return self._root is None

    
    @classmethod
    def _is_red(self, x: Optional[Node[Key, Val]]) -> bool:
        """
        Is node x red?
        :param x: the node to check
        :return: True if node is red False otherwise
        """
        if x is None:
            return False
        return x.color == RedBlackBST.RED

    def _rotate_left(self, h: Node[Key,Val]) -> Node[Key,Val]:
        """
        Make a right-leaning link lean to the left
        :param h:
        :return: The node that has taken h's position
        """
        x = h.right
        assert x is not None #  bc h has a right-leaning (red) link
        h.right = x.left
        x.left = h
        x.color = h.color
        h.color = self.RED
        x.size = h.size
        h.size = self._size(h.left) + self._size(h.right) + 1
        return x

    def _rotate_right(self, h: Node[Key, Val]) -> Node[Key,Val]:
        """
        Make a left-leaning link lean to the right.
        :param h:
        :return: The node that has taken h's position
        """
        x = h.left
        assert x is not None # bc h has a left-leaning (red) link 
        h.left = x.right
        x.right = h
        x.color = h.color
        h.color = self.RED
        x.size = h.size
        h.size = self._size(h.left) + self._size(h.right) + 1
        return x

    def _flip_colors(self, h: Node[Key, Val]) -> None:
        """
        Flip the colors of a node and its two children.
        :param h: the node
        """
        assert h.left is not None
        assert h.right is not None
        h.color = not h.color
        h.left.color = not h.left.color
        h.right.color = not h.right.color

    def _move_red_left(self, h: Node[Key, Val]) -> Node[Key, Val]:
        """
        Assuming that h is red and both h.left and h.left.left
        are black, make h.left or one of its children red.
        Assumes h.right exists
        """
        assert h.right is not None
        self._flip_colors(h)
        if self._is_red(h.right.left):
            h.right = self._rotate_right(h.right)
            h = self._rotate_left(h)
            self._flip_colors(h)
        return h

    def _move_red_right(self, h: Node[Key, Val]) -> Node[Key, Val]:
        """
        Assuming that h is red and both h.right and h.right.left
        are black, make h.right or one of its children red.
        """
        assert h.left is not None # more is true: h.right.left exists and is not red
        self._flip_colors(h)
        if self._is_red(h.left.left):
            h = self._rotate_right(h)
            self._flip_colors(h)
        return h

    def _balance(self, h: Node[Key, Val]) -> Node[Key, Val]:
        """
        Restore red-black tree invariant
        """
        if self._is_red(h.right):
            h = self._rotate_left(h)
        if self._is_red(h.left):
            assert h.left is not None
            if self._is_red(h.left.left):
                h = self._rotate_right(h)
        if self._is_red(h.left) and self._is_red(h.right):
            self._flip_colors(h)
        h.size = self._size(h.left) + self._size(h.right) + 1
        return h

    def height(self) -> int:
        """
        Returns the height of the RedBlackBST
        :return: the height of the RedBlackBST (a 1-node tree has height 0)
        """
        return self._height(self._root)

    def _height(self, x: Optional[Node[Key, Val]]) -> int:
        """
        Returns height of subtree rooted at x.
        """
        if x is None:
            return -1
        return 1 + max(self._height(x.left), self._height(x.right))

    def min(self) -> Key:
        """
        Returns the smallest key in the symbol table.
        :return: the smallest key in the symbol table
        :raises NoSuchElementException: if the symbol table is empty
        """
        if self.is_empty():
            raise NoSuchElementException("calls min() with empty symbol table")
        assert self._root is not None
        return self._min(self._root).key

    def _min(self, x: Node[Key, Val]) -> Node[Key, Val]:
        """
        Returns the Node with the smallest key in subtree rooted at x. None if no such key
        """
        if x.left is None:
            return x
        else:
            return self._min(x.left)

    def max(self) -> Key:
        """
        Returns the largest key in the symbol table.
        :return: the largest key in the symbol table
        :raises NoSuchElementException: if the symbol table is empty
        """
        if self.is_empty():
            raise NoSuchElementException("calls max() with empty symbol table")
        assert self._root is not None
        return self._max(self._root).key

    def _max(self, x: Node[Key, Val]) -> Node[Key, Val]:
        """
        Returns the node with the largest key in subtree rooted at x. None if no such key.
        """
        if x.right is None:
            return x
        else:
            return self._max(x.right)

    def keys(self) -> Queue[Key]:
        """
        Returns all keys in the symbol table.
        :return: all keys in the symbol table
        """
        if self.is_empty():
            return Queue()
        return self.keys_range(self.min(), self.max())

    def keys_range(self, lo: Key, hi: Key) -> Queue[Key]:
        """
        Returns all keys in the symbol table in the given range.
        :param lo: minimum endpoint
        :param hi: maximum endpoint
        :return: all keys in the symbol table between lo (inclusive) and hi (inclusive)
        :raises IllegalArgumentException: if either lo or hi is None
        """
        if lo is None:
            raise IllegalArgumentException("first argument to keys() is None")
        if hi is None:
            raise IllegalArgumentException("second argument to keys() is None")
        queue: Queue[Key] = Queue()
        self._keys(self._root, queue, lo, hi)
        return queue

    def _keys(self, x: Optional[Node[Key, Val]], queue: Queue[Key], lo: Key, hi: Key) -> None:
        """
        Adds the keys between lo and hi in the subtree rooted at x
        to the queue.
        """
        if x is None:
            return
        if lo < x.key:
            self._keys(x.left, queue, lo, hi)
        if not x.key < lo  and x.key <=  hi:
            queue.enqueue(x.key)
        if hi > x.key:
            self._keys(x.right, queue, lo, hi)

    def select(self, k: int) -> Key:
        """
        Return the kth smallest key in the symbol table.
        :param k: the order statistic
        :return: the kth smallest key in the symbol table
        :raises IllegalArgumentException: unless k is between 0 and n-1
        """
        if k < 0 or k >= self.size():
            raise IllegalArgumentException("argument to select() is invalid: {}".format(k))
        assert self._root is not None # bc. 0 <= k < self.size()
        x = self._select(self._root, k)
        return x.key

    def _select(self, x: Node[Key, Val], k: int) -> Node[Key, Val]:
        """
        Returns the node with key of rank k in the subtree rooted at x
        """
        t = self._size(x.left)
        if t > k:
            assert x.left is not None
            return self._select(x.left, k)
        elif t < k:
            assert x.right is not None
            return self._select(x.right, k-t-1)
        else:
            return x

    def rank(self, key: Key) -> int:
        """
        Returns the number of keys in the symbol table strictly less than key.
        :param key: the key
        :return: the number of keys in the symbol table strictly less than key
        :raises IllegalArgumentException: if key is None
        """
        if key is None:
            raise IllegalArgumentException("argument to rank() is None")
        return self._rank(key, self._root)

    def _rank(self, key: Key, x: Optional[Node[Key, Val]]) -> int:
        """
        Returns the number of keys less than key in the subtree rooted at x.
        """
        if x is None:
            return 0
        if key < x.key:
            return self._rank(key, x.left)
        if key > x.key:
            return 1 + self._size(x.left) + self._rank(key, x.right)
        else:
            return self._size(x.left)

    def size_range(self, lo: Key, hi: Key) -> int:
        """
        Returns the number of keys in the symbol table in the given range.
        :param lo: minimum endpoint
        :param hi: maximum endpoint
        :return: the number of keys in the symbol table between lo
        (inclusive) and hi (inclusive)
        :raises IllegalArgumentException: if either lo or hi is None
        """
        if lo is None:
            return IllegalArgumentException("first argument to size() is None")
        if hi is None:
            return IllegalArgumentException("second argument to size() is None")
        if lo > hi:
            return 0
        if self.contains(hi):
            return self.rank(hi) - self.rank(lo) + 1
        else:
            return self.rank(hi) - self.rank(lo)

    def floor(self, key: Key) -> Key:
        """
        Returns the largest key in the symbol table less than or equal to key.
        :param key: the key
        :return: the largest key in the symbol table less than er equal to key
        :raises IllegalArgumentException: if key is None
        :raises NoSuchElementException: if there is no such key
        """
        if key is None:
            raise IllegalArgumentException("argument to floor() is None")
        if self.is_empty():
            raise NoSuchElementException("calls floor() with empty symbol table")
        x = self._floor(self._root, key)
        if x is None:
            raise NoSuchElementException("calls floor() with key < min")
        return x.key

    def _floor(self, x: Optional[Node[Key, Val]], key: Key) -> Optional[Node[Key, Val]]:
        """
        Returns the largest key in the subtree rooted at x less than or equal to the given key.
        """
        if x is None:
            return None
        if key == x.key:
            return x
        if key < x.key:
            return self._floor(x.left, key)
        t = self._floor(x.right, key)
        if t is not None:
            return t
        return x

    def ceiling(self, key: Key) -> Key:
        """
        Returns the smallest key in the symbol table greater than or equal to key.
        :param key: the key
        :return: the smallest key in the symbol table greater than or equal to key
        :raises IllegalArgumentException: if key is None
        :raises NoSuchElementException: if there is no such key
        """
        if key is None:
            raise IllegalArgumentException("argument to ceiling is None")
        if self.is_empty():
            raise NoSuchElementException("calls ceiling() with empty symbol table")
        x = self._ceiling(self._root, key) 
        if x is None:
            raise NoSuchElementException("calls ceiling() with key > max")
        return x.key 

    def _ceiling(self, x: Optional[Node[Key, Val]], key: Key) -> Optional[Node[Key, Val]]:
        """
        Returns the node with the smallest key in the subtree rooted at x greater than or equal to the given key
        """
        if x is None:
            return None
        assert x is not None
        if key == x.key:
            return x
        if key > x.key:
            return self._ceiling(x.right, key)
        t = self._ceiling(x.left, key)
        if t is not None:
            return t
        return x



# Created for BADS 2018
# See README.md for details
# This is python3 
"""
The Shellsort module provides static methods for sorting an
array using shellsort with Knuth's increment sequence (1, 4, 13, 40, ...).
"""
import sys


def sort(a):
    """
    Rearranges the array in ascending order using the natural order.
    :param a: the array to be sorted.
    """
    N = len(a)
    h = 1
    while(h<int(N/3)):
        h = (3*h)+1
    while (h>=1):
        #h-sort the array
        for i in range(h,N):
            #Insert a[i] among a[i-h], a[i-2*h], a[i-3*h]...
            for j in range(i,h-1,-h):
                if not _less(a[j],a[j-h]):
                    break
                _exch(a,j,j-h)
        h = int(h/3)
def _less(v, w):
    return v < w
def _exch(a, i, j):
    t = a[i]
    a[i] = a[j]
    a[j] = t
def _show(a):
    #Prints the array on a single line
    for item in a:
        print(item, end=' ')
    print()
def is_sorted(a):
    """
    Returns true if a is sorted.
    :param a: the array to be checked.
    :returns: True if a is sorted.
    """
    for i in range(1,len(a)):
        if _less(a[i], a[i-1]):
            return False
    return True
def main():
    """
    Reads in a sequence of strings from standard input; Shellsorts them;
    and prints them to standard output in ascending order.
    """
    a = sys.argv[1:]
    sort(a)
    assert is_sorted(a)
    _show(a)

if __name__ == "__main__":
    main()



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
stdstats.py

The stdstats module defines functions related to statistical analysis
and graphical data display.
"""

#-----------------------------------------------------------------------

import math

from itu.algs4.stdlib import stddraw

#-----------------------------------------------------------------------

#def min(a):
#    """
#    Return the minimum value in array a.  Could call the built-in
#    min() function instead.
#    """
#    minumum = float('inf')
#    for x in a:
#        if x < minumum:
#            minumum = x
#    return minumum

#-----------------------------------------------------------------------

#def max(a):
#    """
#    Return the maximum value in array a.  Could call the built-in
#    max() function instead.
#    """
#    maximum = float('-inf')
#    for x in a:
#        if x > maximum:
#            maximum = x
#    return maximum

#-----------------------------------------------------------------------

def mean(a):
    """
    Return the average of the elements of array a.
    """
    return sum(a) / float(len(a))

#-----------------------------------------------------------------------

def var(a):
    """
    Return the sample variance of the elements of array a.
    """
    mu = mean(a)
    total = 0.0
    for x in a:
        total += (x - mu) * (x - mu)
    return total / (float(len(a)) - 1.0)

#-----------------------------------------------------------------------

def stddev(a):
    """
    Return the standard deviation of the elements of array a.
    """
    return math.sqrt(var(a))

#-----------------------------------------------------------------------

def median(a):
    """
    Return the median of the elements of array a.
    """
    b = list(a)  # Make a copy of a.
    b.sort()
    length = len(b)
    if length % 2 == 1:
        return b[length//2]
    else:
        return float(b[length//2 - 1] + b[length//2]) / 2.0

#-----------------------------------------------------------------------

def plotPoints(a):
    """
    Plot the elements of array a as points.
    """
    n = len(a)
    stddraw.setXscale(-1, n)
    stddraw.setPenRadius(1.0 / (3.0 * n))
    for i in range(n):
        stddraw.point(i, a[i])

#-----------------------------------------------------------------------

def plotLines(a):
    """
    Plot the elements of array a as line end-points.
    """
    n = len(a)
    stddraw.setXscale(-1, n)
    stddraw.setPenRadius(0.0)
    for i in range(1, n):
        stddraw.line(i-1, a[i-1], i, a[i])

#-----------------------------------------------------------------------

def plotBars(a):
    """
    Plot the elements of array a as bars.
    """
    n = len(a)
    stddraw.setXscale(-1, n)
    for i in range(n):
        stddraw.filledRectangle(i-0.25, 0.0, 0.5, a[i])

#-----------------------------------------------------------------------

def _main():
    """
    For testing:
    """
    import stdarray
    import stdio

    a = stdarray.readFloat1D()
    #stdio.writef('       min %7.3f\n', min(a))
    #stdio.writef('       max %7.3f\n', max(a))
    stdio.writef('      mean %7.3f\n', mean(a))
    stdio.writef('   std dev %7.3f\n', stddev(a))
    stdio.writef('    median %7.3f\n', median(a))

if __name__ == '__main__':
    _main()



# Created for BADS 2018
# See README.md for details
# This is python3
"""
The Quick3Way module provides static methods for sorting an
array using quicksort with 3-way partitioning.
"""
import sys
from random import shuffle


def sort(a):
    """
    Rearranges the array in ascending order using the natural order.
    :param a: the array to be sorted.
    """
    shuffle(a) #Eliminate dependency on input.
    _sort(a, 0, len(a)-1)

def _sort(a, lo, hi):
    if(hi <= lo):
        return
    lt = lo
    i = lo+1
    gt = hi
    v = a[lo]
    while(i <= gt):
        cmpr = _compare(a[i],v)
        if(cmpr < 0):
            _exch(a,lt,i)
            lt += 1
            i += 1
        elif(cmpr > 0):
            _exch(a, i, gt)
            gt -= 1
        else:
            i += 1
    _sort(a,lo,lt-1)
    _sort(a,gt+1,hi)
    assert is_sorted(a)

def _compare(a,b):
    return (a > b) - (b > a)

def _less(v, w):
    return _compare(v,w) < 0

def _exch(a, i, j):
    t = a[i]
    a[i] = a[j]
    a[j] = t

def _show(a):
    #Prints the array on a single line
    for item in a:
        print(item,end=" ")
    print()

def is_sorted(a):
    """
    Returns true if a is sorted.
    :param a: the array to be checked.
    :returns: True if a is sorted.
    """
    for i in range(1,len(a)):
        if _less(a[i], a[i-1]):
            return False
    return True

def main():
    """
    Reads in a sequence of strings from standard input; Shellsorts them;
    and prints them to standard output in ascending order.
    """
    a = sys.argv[1:]
    sort(a)
    _show(a)

if __name__ == "__main__":
    main()



# Created for BADS 2018
# See README.md for details
# This is python3
import struct
import sys

from itu.algs4.stdlib.binary_stdout import BinaryStdOut


"""
Binary standard input. This class provides methods for reading
in bits from standard input, either one bit at a time (as a boolean),
8 bits at a time (as a char) or 32 bits at a time (as an int)

All primitive types are assumed to be represented in big-endian order.

The client should not mix class to BinaryStdIn with calls to stdin,
otherwise unexpected behavior will result.
"""
class BinaryStdIn:
	EOF = -1
	ins = sys.stdin.buffer
	n = 0
	buffer_ = 0
	is_init = False
	@staticmethod
	def _initialize():
		BinaryStdIn.buffer_ = 0
		BinaryStdIn.n = 0
		BinaryStdIn._fill_buffer()
		BinaryStdIn.is_init = True
	@staticmethod
	def _fill_buffer():
		x = BinaryStdIn.ins.read(1)
		if(x == b''):
			BinaryStdIn.buffer_ = BinaryStdIn.EOF
			BinaryStdIn.n = -1
			return
		BinaryStdIn.buffer_ = struct.unpack('B',x)[0]
		BinaryStdIn.n = 8
	@staticmethod
	def close():
		"""
		Close this input stream and release any associated system resources
		"""
		if(not BinaryStdIn.is_init):
			_initialize
		BinaryStdIn.ins.close()
		BinaryStdIn.is_init = False
	@staticmethod
	def is_empty():
		if(not BinaryStdIn.is_init):
			BinaryStdIn._initialize()
		return BinaryStdIn.buffer_ == BinaryStdIn.EOF
	@staticmethod
	def read_bool():
		if(BinaryStdIn.is_empty()):
			raise EOFError("Reading from empty input stream")
		BinaryStdIn.n -= 1
		bit = ((BinaryStdIn.buffer_ >> BinaryStdIn.n) & 1) == 1
		if(BinaryStdIn.n == 0):
			BinaryStdIn._fill_buffer()
		return bit
	@staticmethod
	def read_char():
		if(BinaryStdIn.is_empty()):
			raise EOFError("Reading from empty input stream")
		if(BinaryStdIn.n==8):
			x = BinaryStdIn.buffer_
			BinaryStdIn._fill_buffer()
			return chr(x & 0xff)
		x = BinaryStdIn.buffer_
		x <<= (8-BinaryStdIn.n)
		oldN = BinaryStdIn.n
		if(BinaryStdIn.is_empty()):
			raise EOFError("Reading from empty input stream")
		BinaryStdIn._fill_buffer()
		BinaryStdIn.n = oldN
		x |= (BinaryStdIn.buffer_>> BinaryStdIn.n)
		return chr(x & 0xff)
	@staticmethod
	def read_string():
		if(BinaryStdIn.is_empty()):
			raise EOFError("Reading from empty input stream")
		sb = ""
		while(not BinaryStdIn.is_empty()):
			sb += BinaryStdIn.read_char()
		return sb

	@staticmethod
	def read_int(r=32):
		if(r==32):
			x = 0
			for i in range(0,4):
				c = BinaryStdIn.read_char()
				x <<=8
				x |= ord(c)
			return x
		if(r<1 or r>32):
			raise ValueError("Illegal value for r = {}".format(r))
		x = 0
		for i in range(0,r):
			x <<= 1
			bit = BinaryStdIn.read_bool()
			if(bit):
				x |= 1
		return x

def main():
	while(not BinaryStdIn.is_empty()):
		BinaryStdOut.write_char(BinaryStdIn.read_char())
if __name__ == '__main__':
	main()



# Created for BADS 2018
# see README.md for details
# This is python3 

import math

from itu.algs4.fundamentals.queue import Queue
from itu.algs4.fundamentals.uf import UF
from itu.algs4.sorting.min_pq import MinPQ


class LazyPrimMST:
    """
    The LazyPrimMST class represents a data type for computing a
    minimum spanning tree in an edge-weighted graph.
    The edge weights can be positive, zero, or negative and need not
    be distinct. If the graph is not connected, it computes a minimum
    spanning forest, which is the union of minimum spanning trees
    in each connected component. The weight() method returns the 
    weight of a minimum spanning tree and the edges() method
    returns its edges.

    This implementation uses a lazy version of Prim's algorithm
    with a binary heap of edges.
    The constructor takes time proportional to E log E
    and extra space (not including the graph) proportional to E,
    where V is the number of vertices and E is the number of edges.
    Afterwards, the weight() method takes constant time
    and the edges() method takes time proportional to V.
    """
    FLOATING_POINT_EPSILON = 1E-12

    def __init__(self,  G):
        """
        Compute a minimum spanning tree (or forest) of an edge-weighted graph.
        :param G: the edge-weighted graph
        """
        self._weight = 0.0                  # total weight of MST
        self._mst = Queue()                 # edges in the MST        
        self._marked = [False] * G.V()      # marked[v] = True if v on tree
        self._pq = MinPQ()                  # edges with one endpoint in tree

        for v in range(G.V()):              # run Prim from all vertices to
            if not self._marked[v]: 
                self._prim(G, v)            # get a minimum spanning forest

        # check optimality conditions
        assert self._check(G)

    def _prim(self, G, s):
        # run Prim's algorithm
        self._scan(G, s)
        while not self._pq.is_empty():                  # better to stop when mst has V-1 edges
            e = self._pq.del_min()                      # smallest edge on pq
            v = e.either()                              # two endpoints
            w = e.other(v)                              
            assert self._marked[v] or self._marked[w]
            if self._marked[v] and self._marked[w]:     # lazy, both v and w already scanned
                continue
            self._mst.enqueue(e)                        # add e to MST
            self._weight += e.weight()
            if not self._marked[v]: self._scan(G, v)    # v becomes part of tree
            if not self._marked[w]: self._scan(G, w)    # w becomes part of tree
        
    
    def _scan(self, G, v):
        # add all edges e incident to v onto pq if the other endpoint has not yet been scanned
        assert not self._marked[v]
        self._marked[v] = True
        for e in G.adj(v):
            if not self._marked[e.other(v)]: 
                self._pq.insert(e)
    
    def edges(self): 
        """
        Returns the edges in a minimum spanning tree (or forest).
        :returns: the edges in a minimum spanning tree (or forest) as
            an iterable of edges
        """
        return self._mst

    def weight(self):
        """
        Returns the sum of the edge weights in a minimum spanning tree (or forest).
        :returns: the sum of the edge weights in a minimum spanning tree (or forest)
        """    
        return self._weight
    
    def _check(self, G):
        # check optimality conditions (takes time proportional to E V lg* V)
        
        totalWeight = 0.0 # check weight
        for e in self.edges():
            totalWeight += e.weight()
        
        if abs(totalWeight - self.weight()) > LazyPrimMST.FLOATING_POINT_EPSILON:
            error = "Weight of edges does not equal weight(): {} vs. {}\n".format(totalWeight, self.weight())           
            print(error, file=sys.stderr)
            return False

        # check that it is acyclic
        uf = UF(G.V())
        for e in self.edges():
            v = e.either()
            w = e.other(v)
            if uf.connected(v, w):
                print("Not a forest", file=sys.stderr)
                return False            
            uf.union(v, w)
        
        # check that it is a spanning forest
        for e in G.edges():
            v = e.either()
            w = e.other(v)
            if not uf.connected(v, w):
                print("Not a forest", file=sys.stderr)
                return False

        # check that it is a minimal spanning forest (cut optimality conditions)
        for e in self.edges():
            # all edges in MST except e
            uf = UF(G.V())
            for f in self._mst:
                x = f.either()
                y = f.other(x)
                if f != e: 
                    uf.union(x, y)            

            # check that e is min weight edge in crossing cut
            for f in G.edges():
                x = f.either()
                y = f.other(x)
                if not uf.connected(x, y):
                    if f.weight() < e.weight():
                        error = "Edge {} violates cut optimality conditions".format(f)
                        print(error, file=sys.stderr)
                        return False
        return True

if __name__ == "__main__":
    import sys
    from itu.algs4.stdlib.instream import InStream
    from itu.algs4.stdlib import stdio
    from itu.algs4.graphs.edge_weighted_graph import EdgeWeightedGraph
    
    In = InStream(sys.argv[1])
    G = EdgeWeightedGraph.from_stream(In)
    mst = LazyPrimMST(G)
    for e in mst.edges():
        stdio.writeln(e)    
    stdio.writef("%.5f\n", mst.weight())



# Created for BADS 2018
# See README.md for details
# This is python3 
"""
The Insertion module provides static methods for sorting an
array using insertion sort.

This implementation makes ~ 1/2 n^2 compares and exchanges in
the worst case, so it is not suitable for sorting large arbitrary arrays.
More precisely, the number of exchanges is exactly equal to the number
of inversions. So, for example, it sorts a partially-sorted array
in linear time.
The sorting algorithm is stable and uses O(1) extra memory.
 """
import sys
from typing import List, TypeVar

T = TypeVar('T')

def sort(a: List[T]):
	"""
	Rearranges the array in ascending order, using the natural order.
	:param a: the array to be sorted.
	"""
	#Sort a[] into increasing order.
	N = len(a)
	for i in range(1,N):
		#Insert a[i] among a[i-1], a[i-2], a[i-3]...
		for j in range(i,0,-1):
			if not _less(a[j], a[j-1]):
				break
			_exch(a,j,j-1)
			
def _less(v: T, w: T):
	return v < w

def _exch(a: List[T], i: int, j: int):
	t = a[i]
	a[i] = a[j]
	a[j] = t
	
def _show(a: List[T]):
	#Prints the array on a single line
	for item in a:
		print(item, end=' ')
	print()
	
def is_sorted(a: List[T]):
	"""
	Returns true if a is sorted.
	:param a: the array to be checked.
	:returns: True if a is sorted.
	"""
	for i in range(1,len(a)):
		if _less(a[i], a[i-1]):
			return False
	return True

def main():
	"""
	Reads in a sequence of strings from standard input; Shellsorts them;
	and prints them to standard output in ascending order.
	"""
	a = sys.argv[1:]
	sort(a)
	assert is_sorted(a)
	_show(a)
	
if __name__ == "__main__":
	main()



# Created for BADS 2018
# See README.md for details
# Python 3
import sys

from itu.algs4.stdlib import stdio

#  Execution:    python file_index.py file1.txt file2.txt file3.txt ...
#
#  % python file_index.py ex*.txt
#  age
#   ex3.txt
#   ex4.txt 
# best
#   ex1.txt 
# was
#   ex1.txt
#   ex2.txt
#   ex3.txt
#   ex4.txt 
#
#  % python file_index.py *.txt
#
#  % python file_index.py *.py


"""
 *  The {@code FileIndex} class provides a client for indexing a set of files,
 *  specified as command-line arguments. It takes queries from standard input
 *  and prints each file that contains the given query.
"""

# key = word, value = set of files containing that word
if __name__ == '__main__':
    st = {} 
    args = sys.argv[1:]

    # create inverted index of all files
    print("Indexing files")
    for filename in args:
        print("  " + filename)
        file = open(filename, 'r')
        for line in file.readlines():
            for word in line.split():
                if not word in st:
                    st[word] = set()
                s = st.get(word)
                s.add(file)

    # read queries from standard input, one per line
    while not stdio.isEmpty():
        query = stdio.readString()
        if query in st:
            s = st.get(query)
            for file in s:
                print(" " + file.name)



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
stdio.py

The stdio module supports reading from standard input and writing to
sys.stdout.

Note:  Usually it's a bad idea to mix these three sets of reading
functions:

-- isEmpty(), readInt(), readFloat(), readBool(), readString()

-- hasNextLine(), readLine()

-- readAll(), readAllInts(), readAllFloats(), readAllBools(),
   readAllStrings(), readAllLines()

Usually it's better to use one set exclusively.
"""
    
import re
import sys

#-----------------------------------------------------------------------

# Change sys.stdin so it provides universal newline support. 

if (sys.hexversion < 0x03000000):
    import os
    sys.stdin = os.fdopen(sys.stdin.fileno(), 'rU', 0)
else:    
    sys.stdin = open(sys.stdin.fileno(), 'r', newline=None)

#=======================================================================
# print to stderr
# from https://stackoverflow.com/questions/5574702/how-to-print-to-stderr-in-python#14981125
#=======================================================================

def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)
    
#=======================================================================
# Writing functions
#=======================================================================

def writeln(x=''):
    """
    Write x and an end-of-line mark to standard output.
    """
    if sys.hexversion < 0x03000000:
        x = unicode(x)
        x = x.encode('utf-8')
    else:
        x = str(x)
    sys.stdout.write(x)
    sys.stdout.write('\n')
    sys.stdout.flush()

#-----------------------------------------------------------------------

def write(x=''):
    """
    Write x to standard output.
    """
    if (sys.hexversion < 0x03000000):
        x = unicode(x)
        x = x.encode('utf-8')
    else:
        x = str(x)
    sys.stdout.write(x)
    sys.stdout.flush()

#-----------------------------------------------------------------------

def writef(fmt, *args):
    """
    Write each element of args to standard output.  Use the format
    specified by string fmt.
    """
    x = fmt % args
    if sys.hexversion < 0x03000000:
        x = unicode(x)
        x = x.encode('utf-8')
    sys.stdout.write(x)
    sys.stdout.flush()

#=======================================================================
# Reading functions
#=======================================================================

_buffer = ''

#-----------------------------------------------------------------------

def _readRegExp(regExp):
    """
    Discard leading white space characters from standard input. Then read
    from standard input and return a string matching regular expression
    regExp.  Raise an EOFError if no non-whitespace characters remain
    in standard input.  Raise a ValueError if the next characters to
    be read from standard input do not match 'regExp'.
    """
    global _buffer
    if isEmpty():
        raise EOFError()
    compiledRegExp = re.compile(r'^\s*' + regExp)
    match = compiledRegExp.search(_buffer)
    if match is None:
        raise ValueError()
    s = match.group()
    _buffer = _buffer[match.end():]
    return s.lstrip()

#-----------------------------------------------------------------------

def isEmpty():
    """
    Return True if no non-whitespace characters remain in standard
    input. Otherwise return False.
    """
    global _buffer
    while _buffer.strip() == '':
        line = sys.stdin.readline()
        if sys.hexversion < 0x03000000:
            line = line.decode('utf-8')
        if line == '':
            return True
        _buffer += line
    return False

#-----------------------------------------------------------------------

def readInt():
    """
    Discard leading white space characters from standard input. Then
    read from standard input a sequence of characters comprising an
    integer. Convert the sequence of characters to an integer, and
    return the integer.  Raise an EOFError if no non-whitespace
    characters remain in standard input. Raise a ValueError if the
    next characters to be read from standard input cannot comprise
    an integer.
    """
    s = _readRegExp(r'[-+]?(0[xX][\dA-Fa-f]+|0[0-7]*|\d+)')
    radix = 10
    strLength = len(s)
    if (strLength >= 1) and (s[0:1] == '0'): radix = 8
    if (strLength >= 2) and (s[0:2] == '-0'): radix = 8
    if (strLength >= 2) and (s[0:2] == '0x'): radix = 16
    if (strLength >= 2) and (s[0:2] == '0X'): radix = 16
    if (strLength >= 3) and (s[0:3] == '-0x'): radix = 16
    if (strLength >= 3) and (s[0:3] == '-0X'): radix = 16
    return int(s, radix)

#-----------------------------------------------------------------------

def readAllInts():
    """
    Read all remaining strings from standard input, convert each to
    an int, and return those ints in an array. Raise a ValueError if
    any of the strings cannot be converted to an int.
    """
    strings = readAllStrings()
    ints = []
    for s in strings:
        i = int(s)
        ints.append(i)
    return ints

#-----------------------------------------------------------------------

def readFloat():
    """
    Discard leading white space characters from standard input. Then
    read from standard input a sequence of characters comprising a
    float. Convert the sequence of characters to a float, and return the
    float.  Raise an EOFError if no non-whitespace characters remain
    in standard input. Raise a ValueError if the next characters to be
    read from standard input cannot comprise a float.
    """
    s = _readRegExp(r'[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?')
    return float(s)

#-----------------------------------------------------------------------

def readAllFloats():
    """
    Read all remaining strings from standard input, convert each to
    a float, and return those floats in an array. Raise a ValueError if
    any of the strings cannot be converted to a float.
    """
    strings = readAllStrings()
    floats = []
    for s in strings:
        f = float(s)
        floats.append(f)
    return floats

#-----------------------------------------------------------------------

def readBool():
    """
    Discard leading white space characters from standard input. Then
    read from standard input a sequence of characters comprising a bool.
    Convert the sequence of characters to a bool, and return the
    bool.  Raise an EOFError if no non-whitespace characters remain
    in standard input. Raise a ValueError if the next characters to be
    read from standard input cannot comprise a bool.

    These character sequences can comprise a bool:
    -- True
    -- False
    -- 1 (means true)
    -- 0 (means false)
    """
    s = _readRegExp(r'(True)|(False)|1|0')
    if (s == 'True') or (s == '1'):
        return True
    return False

#-----------------------------------------------------------------------

def readAllBools():
    """
    Read all remaining strings from standard input, convert each to
    a bool, and return those bools in an array. Raise a ValueError if
    any of the strings cannot be converted to a bool.
    """
    strings = readAllStrings()
    bools = []
    for s in strings:
        b = bool(s)
        bools.append(b)
    return bools

#-----------------------------------------------------------------------

def readString():
    """
    Discard leading white space characters from standard input. Then
    read from standard input a sequence of characters comprising a
    string, and return the string. Raise an EOFError if no
    non-whitespace characters remain in standard input.
    """
    s = _readRegExp(r'\S+')
    return s

#-----------------------------------------------------------------------

def readAllStrings():
    """
    Read all remaining strings from standard input, and return them in
    an array.
    """
    strings = []
    while not isEmpty():
        s = readString()
        strings.append(s)
    return strings

#-----------------------------------------------------------------------

def hasNextLine():
    """
    Return True if standard input has a next line. Otherwise return
    False.
    """
    global _buffer
    if _buffer != '':
        return True
    else:
        _buffer = sys.stdin.readline()
        if sys.hexversion < 0x03000000:
            _buffer = _buffer.decode('utf-8')
        if _buffer == '':
            return False
        return True

#-----------------------------------------------------------------------

def readLine():
    """
    Read and return as a string the next line of standard input.
    Raise an EOFError is there is no next line.
    """
    global _buffer
    if not hasNextLine():
        raise EOFError()
    s = _buffer
    _buffer = ''
    return s.rstrip('\n')

#-----------------------------------------------------------------------

def readAllLines():
    """
    Read all remaining lines from standard input, and return them as
    strings in an array.
    """
    lines = []
    while hasNextLine():
        line = readLine()
        lines.append(line)
    return lines

#-----------------------------------------------------------------------

def readAll():
    """
    Read and return as a string all remaining lines of standard input.
    """
    global _buffer
    s = _buffer
    _buffer = ''
    for line in sys.stdin:
        if sys.hexversion < 0x03000000:
            line = line.decode('utf-8')
        s += line
    return s

#=======================================================================
# For Testing
#=======================================================================

def _testWrite():
    writeln()
    writeln('string')
    writeln(123456)
    writeln(123.456)
    writeln(True)
    write()
    write('string')
    write(123456)
    write(123.456)
    write(True)
    writeln()
    writef('<%s> <%8d> <%14.8f>\n', 'string', 123456, 123.456)
    writef('formatstring\n')

#-----------------------------------------------------------------------

def _main():
    """
    For testing. The command-line argument should be the name of the
    function that should be called.
    """

    map = {
        'readInt':    readInt,    'readAllInts':    readAllInts,
        'readFloat':  readFloat,  'readAllFloats':  readAllFloats,
        'readBool':   readBool,   'readAllBools':   readAllBools,
        'readString': readString, 'readAllStrings': readAllStrings,
        'readLine':   readLine,   'readAllLines' :  readAllLines,
        'readAll':    readAll }

    testId = sys.argv[1]

    if testId == 'write':
        _testWrite()
    else:
        writeln(map[testId]())

if __name__ == '__main__':
    _main()



# Created for BADS 2018
# See README.md for details
# Python 3

from typing import Generic, Iterator, List, Optional, Sequence, TypeVar

T = TypeVar('T')

class Node(Generic[T]):
    # helper linked list class
    def __init__(self):
        self.item: T = None
        self.next: Optional[Node] = None

class Stack(Generic[T]):
    """
    The Stack class represents a last-in-first-out (LIFO) stack of generic items.
    It supports the usual push and pop operations, along with methods
    for peeking at the top item, testing if the stack is empty, and iterating through
    the items in LIFO order.

    This implementation uses a singly linked list with a static nested class for
    linked-list nodes. See LinkedStack for the version from the
    textbook that uses a non-static nested class.
    See ResizingArrayStack for a version that uses a resizing array.
    The push, pop, peek, size, and is-empty
    operations all take constant time in the worst case.
    """

    def __init__(self) -> None:
        """
        Initializes an empty stack.
        """
        self._first: Optional[Node[T]] = None
        self._n: int = 0

    def is_empty(self) -> bool:
        """
        Returns true if this stack is empty.

        :returns: true if this stack is empty false otherwise
        """
        return self._n == 0

    def size(self) -> int:
        """
        Returns the number of items in this stack.

        :returns: the number of items in this stack
        """
        return self._n

    def __len__(self) -> int:
        return self.size()

    def push(self, item: T) -> None:
        """
        Adds the item to this stack.

        :param item: the item to add
        """
        oldfirst = self._first
        self._first = Node()
        self._first.item = item
        self._first.next = oldfirst
        self._n += 1

    def pop(self) -> T:
        """
        Removes and returns the item most recently added to this stack.

        :returns: the item most recently added
        :raises ValueError: if this stack is empty
        """
        if self.is_empty(): 
            raise ValueError("Stack underflow")
        assert self._first is not None
        item = self._first.item
        assert item is not None
        self._first = self._first.next
        self._n -= 1
        return item


    def peek(self) -> T:
        """
        Returns (but does not remove) the item most recently added to this stack.

        :returns: the item most recently added to this stack
        :raises ValueError: if this stack is empty
        """
        if self.is_empty(): 
            raise ValueError("Stack underflow")
        assert self._first is not None
        item = self._first.item
        assert item is not None
        return item

    def __repr__(self) -> str:
        """
        Returns a string representation of this stack.

        :returns: the sequence of items in this stack in LIFO order, separated by spaces
        """
        s = []
        for item in self:
            s.append(item.__repr__())
        return " ".join(s)

    def __iter__(self) -> Iterator[T]:
        """
        Returns an iterator to this stack that iterates through the items in LIFO order.

        :return: an iterator to this stack that iterates through the items in LIFO order
        """
        current = self._first
        while current is not None:
            item = current.item
            assert item is not None
            yield item
            current = current.next

class FixedCapacityStack(Generic[T]):
    def __init__(self, capacity: int):
        self.a: List[Optional[T]] = [None] * capacity
        self.n: int = 0

    def is_empty(self) -> bool:
        return self.n == 0

    def size(self) -> int:
        return self.n

    def __len__(self) -> int:
        return self.size()

    def push(self, item: T):
        self.a[self.n] = item
        self.n += 1

    def pop(self) -> T:
        self.n -= 1
        item = self.a[self.n]
        assert item is not None
        return item


class ResizingArrayStack(Generic[T]):
    def __init__(self) -> None:
        self.a: List[Optional[T]] = [None]
        self.n: int = 0

    def is_empty(self) -> bool:
        return self.n == 0

    def size(self) -> int:
        return self.n

    def __len__(self) -> int:
        return self.size()

    def resize(self, capacity: int) -> None:
        temp: List[Optional[T]] = [None] * capacity
        for i in range(self.n):
            temp[i] = self.a[i]
        self.a = temp

    def push(self, item: T) -> None:
        if self.n == len(self.a):
            self.resize(2 * len(self.a))
        self.a[self.n] = item
        self.n += 1

    def pop(self) -> T:
        self.n -= 1
        item = self.a[self.n]
        self.a[self.n] = None
        if self.n > 0 and self.n <= len(self.a)//4:
            self.resize(len(self.a)//2)
        assert item is not None
        return item

    def __iter__(self) -> Iterator[T]:
        i = self.n -1
        while i >= 0:
            item = self.a[i]
            assert item is not None
            yield item
            i -= 1



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
stdaudio.py

The stdaudio module defines functions related to audio.
"""

#-----------------------------------------------------------------------

import sys

import numpy

import pygame

#-----------------------------------------------------------------------

_SAMPLES_PER_SECOND = 44100
_SAMPLE_SIZE = -16           # Each sample is a signed 16-bit int
_CHANNEL_COUNT = 1           # 1 => mono, 2 => stereo
_AUDIO_BUFFER_SIZE = 1024    # In number of samples
_CHECK_RATE = 44100          # How often to check the queue

_myBuffer = []
_MY_BUFFER_MAX_LENGTH = 4096 # Determined experimentally.

def wait():
    """
    Wait for the sound queue to become empty.  Informally, wait for the
    currently playing sound to finish.
    """

    # Can have at most one sound in the queue.  So must wait for the
    # queue to become empty before adding a new sound to the queue.

    global _channel
    clock = pygame.time.Clock()
    while _channel.get_queue() is not None:
    #while pygame.mixer.get_busy():
        clock.tick(_CHECK_RATE)

def playSample(s):
    """
    Play sound sample s.
    """
    global _myBuffer
    global _channel
    _myBuffer.append(s)
    if len(_myBuffer) > _MY_BUFFER_MAX_LENGTH:
        temp = []
        for sample in _myBuffer:
            temp.append(numpy.int16(sample * float(0x7fff)))
        samples = numpy.array(temp, numpy.int16)
        sound = pygame.sndarray.make_sound(samples)
        wait()
        _channel.queue(sound)
        _myBuffer = []

def playSamples(a):
    """
    Play all sound samples in array a.
    """
    for sample in a:
        playSample(sample)

def playArray(a):
    """
    This function is deprecated. It has the same behavior as
    stdaudio.playSamples(). Please call stdaudio.playSamples() instead.
    """
    playSamples(a)

def playFile(f):
    """
    Play all sound samples in the file whose name is f.wav.
    """
    a = read(f)
    playSamples(a)
    #sound = pygame.mixer.Sound(fileName)
    #samples = pygame.sndarray.samples(sound)
    #wait()
    #sound.play()

def save(f, a):
    """
    Save all samples in array a to the WAVE file whose name is f.wav.
    """

    # Saving to a WAV file isn't handled by PyGame, so use the
    # standard "wave" module instead.

    import wave
    fileName = f + '.wav'
    temp = []
    for sample in a:
        temp.append(int(sample * float(0x7fff)))
    samples = numpy.array(temp, numpy.int16)
    file = wave.open(fileName, 'w')
    file.setnchannels(_CHANNEL_COUNT)
    file.setsampwidth(2)  # 2 bytes
    file.setframerate(_SAMPLES_PER_SECOND)
    file.setnframes(len(samples))
    file.setcomptype('NONE', 'descrip')  # No compression
    file.writeframes(samples.tostring())
    file.close()

def read(f):
    """
    Read all samples from the WAVE file whose names is f.wav.
    Store the samples in an array, and return the array.
    """
    fileName = f + '.wav'
    sound = pygame.mixer.Sound(fileName)
    samples = pygame.sndarray.samples(sound)
    temp = []
    for i in range(len(samples)):
        temp.append(float(samples[i]) / float(0x7fff))
    return temp

# Initialize PyGame to handle audio.
try:
    pygame.mixer.init(_SAMPLES_PER_SECOND, _SAMPLE_SIZE,
        _CHANNEL_COUNT, _AUDIO_BUFFER_SIZE)
    _channel = pygame.mixer.Channel(0)
except pygame.error:
    stdio.writeln('Could not initialize PyGame')
    sys.exit(1)

#-----------------------------------------------------------------------

def _createTextAudioFile():
    """
    For testing. Create a text audio file.
    """
    notes = [
        7, .270,
        5, .090,
        3, .180,
        5, .180,
        7, .180,
        6, .180,
        7, .180,
        3, .180,
        5, .180,
        5, .180,
        5, .180,
        5, .900,

        5, .325,
        3, .125,
        2, .180,
        3, .180,
        5, .180,
        4, .180,
        5, .180,
        2, .180,
        3, .180,
        3, .180,
        3, .180,
        3, .900,
        ]

    from itu.algs4.stdlib import outstream
    outStream = outstream.OutStream('looney.txt')
    for note in notes:
        outStream.writeln(note)

def _main():
    """
    For testing.
    """
    import os
    import math
    from itu.algs4.stdlib import stdio, instream

    _createTextAudioFile()

    stdio.writeln('Creating and playing in small chunks...')
    sps = _SAMPLES_PER_SECOND
    inStream = instream.InStream('looney.txt')
    while not inStream.isEmpty():
        pitch = inStream.readInt()
        duration = inStream.readFloat()
        hz = 440 * math.pow(2, pitch / 12.0)
        N = int(sps * duration)
        notes = []
        for i in range(N+1):
            notes.append(math.sin(2*math.pi * i * hz / sps))
        playSamples(notes)
    wait()

    stdio.writeln('Creating and playing in one large chunk...')
    sps = _SAMPLES_PER_SECOND
    notes = []
    inStream = instream.InStream('looney.txt')
    while not inStream.isEmpty():
        pitch = inStream.readInt()
        duration = inStream.readFloat()
        hz = 440 * math.pow(2, pitch / 12.0)
        N = int(sps * duration)
        for i in range(N+1):
            notes.append(math.sin(2*math.pi * i * hz / sps))
    playSamples(notes)
    wait()

    stdio.writeln('Saving...')
    save('looney', notes)

    stdio.writeln('Reading...')
    notes = read('looney')

    stdio.writeln('Playing an array...')
    playSamples(notes)
    wait()

    stdio.writeln('Playing a file...')
    playFile('looney')
    wait()

    os.remove('looney.wav')
    os.remove('looney.txt')

if __name__ == '__main__':
    _main()



# Created for BADS 2018
# See README.md for details
# This is python3 

import sys

from itu.algs4.fundamentals.bag import Bag
from itu.algs4.fundamentals.stack import Stack
from itu.algs4.graphs.digraph import Digraph
from itu.algs4.graphs.directed_dfs import DirectedDFS


class NFA:
	"""
	The NFA class provides a data type for creating a
	nondeterministic finite state automaton (NFA) from a regular
	expression and testing whether a given string is matched by that regular
	expression.
	It supports the following operations: concatenation, closure, binary or,
	and parentheses, metacharacters (either in the text or pattern),
	capturing capabilities, greedy or reluctant/lazy modifiers,
	and other features in industrial-strength implementations
	such as Java's Pattern and Matcher.

	This implementation builds the NFA using a digraph and a stack
	and simulates the NFA using digraph search (see the textbook for details).
	The constructor takes time proportional to m, where m is the
	number of characters in the regular expression.
	The recognizes() method takes time proportional to m*n, where n
	is the number of characters in the text.

	For additional documentation, see section 5.4 of Algorithms, 4th Edition
	by Robert Sedgewick and Kevin Wayne.
	"""
	def __init__(self, regex):
		"""
		Initializes the NFA from the specified regular expression.

		:param regex: the regular expression
		"""
		self.regex = regex
		m = len(regex)
		self.m = m
		ops = Stack()
		graph = Digraph(m+1)
		for i in range(0,m):
			lp = i
			if(regex[i] == '(' or regex[i] == '|'):
				ops.push(i)
			elif(regex[i] == ')'):
				or_ = ops.pop()

				# 2-way or operator
				if(regex[or_] == '|'):
					lp = ops.pop()
					graph.add_edge(lp, or_+1)
					graph.add_edge(or_, i)
				elif(regex[or_] == '('):
					lp = or_
				else:
					assert False
			if(i < m-1 and regex[i+1] == '*'):
				graph.add_edge(lp, i+1)
				graph.add_edge(i+1, lp)
			if(regex[i] == '(' or regex[i] == '*' or regex[i] == ')'):
				graph.add_edge(i, i+1)
		if(ops.size() != 0):
			raise ValueError("Invalid regular expression")
		self.graph = graph

	def recognizes(self, txt):
		"""
		Returns True if the text is matched by the regular expression.

		:param txt: the text
		:returns: True if the text is matched by the regular expression;
					False otherwise
		"""
		regex = self.regex
		graph = self.graph
		m = self.m
		dfs = DirectedDFS(graph, 0)
		pc = Bag()
		for v in range(0, graph.V()):
			if(dfs.is_marked(v)):
				pc.add(v)

		#Compute possible NFA states for txt[i+1]
		for i in range(0, len(txt)):
			if(txt[i] == '*' or txt[i] == '|' or txt[i] == '(' or txt[i] == ')'):
				raise ValueError("text contains the metacharacter '{}'".format(txt[i]))
			match = Bag()
			for v in pc:
				if(v == m):
					continue
				if(regex[v] == txt[i] or regex[i] == '.'):
					match.add(v+1)
			dfs = DirectedDFS(graph, *match)
			pc = Bag()
			for v in range(0, graph.V()):
				if(dfs.is_marked(v)):
					pc.add(v)
			# Optimization if no states reachable
			if(pc.size() == 0):
				return False
		for v in pc:
			if(v == m):
				return True
			return False
def main():
	"""
	Unit tests the NFA data type
	"""
	regex = "({})".format(sys.argv[1])
	txt = sys.argv[2]
	nfa = NFA(regex)
	print(nfa.recognizes(txt))
if __name__ == '__main__':
	main()



from itu.algs4.graphs.breadth_first_paths import BreadthFirstPaths
from itu.algs4.graphs.symbol_graph import SymbolGraph
from itu.algs4.stdlib import stdio


class DegreesOfSeparation:
    """
    The DegreesOfSeparation class provides a client for finding
    the degree of separation between one distinguished individual and
    every other individual in a social network.
    As an example, if the social network consists of actors in which
    two actors are connected by a link if they appeared in the same movie,
    and Kevin Bacon is the distinguished individual, then the client
    computes the Kevin Bacon number of every actor in the network.

    The running time is proportional to the number of individuals and
    connections in the network. If the connections are given implicitly,
    as in the movie network example (where every two actors are connected
    if they appear in the same movie), the efficiency of the algorithm
    is improved by allowing both movie and actor vertices and connecting
    each movie to all of the actors that appear in that movie.
    """
    
    def __init__(self):
        """this class shouldn't be instantiated"""
        pass


    @staticmethod
    def main(args):
        """
        Reads in a social network from a file, and then repeatedly reads in
        individuals from standard input and prints out their degrees of
        separation.
        Takes three command-line arguments: the name of a file,
        a delimiter, and the name of the distinguished individual.
        Each line in the file contains the name of a vertex, followed by a
        list of the names of the vertices adjacent to that vertex,
        separated by the delimiter.
        
        :param args: the command-line arguments
        """
        filename  = args[1]
        delimiter = args[2]
        source = args[3]

        sg = SymbolGraph(filename, delimiter)
        G = sg.graph()
        if not sg.contains(source):
            stdio.writeln("{} not in database".format(source))
            return
        
        s = sg.index_of(source)
        bfs = BreadthFirstPaths(G, s)

        while not stdio.isEmpty():
            sink = stdio.readLine()
            if sg.contains(sink):
                t = sg.index_of(sink)
                if bfs.has_path_to(t):
                    for v in bfs.path_to(t):
                      stdio.writef("\t%s\n", sg.name_of(v))
                else:
                    stdio.writeln("\tNot connected")
            else:
                stdio.writeln("\tNot in database.")
        
if __name__ == "__main__":
    import sys
    DegreesOfSeparation.main(sys.argv)



# Created for BADS 2018
# See README.md for details
# Python 3

"""
 *  Execution:    python transitive_closure.py filename.txt
 *  Dependencies: Digraph DirectedDFS 
 *  Data files:   https:#algs4.cs.princeton.edu/42digraph/tinyDG.txt
 *
 *  Compute transitive closure of a digraph and support
 *  reachability queries.
 *
 *  Preprocessing time: O(V(E + V)) time.
 *  Query time: O(1).
 *  Space: O(V^2).
 *
 *  % python transitive_closure.py tinyDG.txt
 *         0  1  2  3  4  5  6  7  8  9 10 11 12
 *  --------------------------------------------
 *    0:   T  T  T  T  T  T                     
 *    1:      T                                 
 *    2:   T  T  T  T  T  T                     
 *    3:   T  T  T  T  T  T                     
 *    4:   T  T  T  T  T  T                     
 *    5:   T  T  T  T  T  T                     
 *    6:   T  T  T  T  T  T  T        T  T  T  T
 *    7:   T  T  T  T  T  T  T  T  T  T  T  T  T
 *    8:   T  T  T  T  T  T  T  T  T  T  T  T  T
 *    9:   T  T  T  T  T  T           T  T  T  T
 *   10:   T  T  T  T  T  T           T  T  T  T
 *   11:   T  T  T  T  T  T           T  T  T  T
 *   12:   T  T  T  T  T  T           T  T  T  T
 *
"""
import sys

from itu.algs4.errors.errors import IllegalArgumentException
from itu.algs4.graphs.digraph import Digraph
from itu.algs4.graphs.directed_dfs import DirectedDFS
from itu.algs4.stdlib.instream import InStream


class TransitiveClosure:
    """
     * Computes the transitive closure of the digraph G.
     * @param G the digraph
     """
    def __init__(self, G): 
        self._tc = [None] * G.V()  # tc[v] = reachable from v
        for v in range(G.V()):
            self._tc[v] = DirectedDFS(G, v)
    """
     * Is there a directed path from vertex v to vertex w in the digraph?
     * @param  v the source vertex
     * @param  w the target vertex
     * @return true if there is a directed path from v to w,
     *         false otherwise
     * @throws IllegalArgumentException unless 0 <= v < V
     * @throws IllegalArgumentException unless 0 <= w < V
     """
    def reachable(self, v, w):
        self._validate_vertex(v)
        self._validate_vertex(w)
        return self._tc[v].is_marked(w)

    # throw an IllegalArgumentException unless 0 <= v < V
    def _validate_vertex(self, v):
        V = len(self._tc)
        if v < 0 or v >= V:
            raise IllegalArgumentException("vertex {} is not between 0 and {}".format(v,V-1))
    

def main(args):
    stream = InStream(args[0])
    G = Digraph.from_stream(stream)

    tc = TransitiveClosure(G)

    # print header
    print("     ", end='')
    for v in range(G.V()):
        print("{x:3d}".format(x=v), end='')
    print()
    print("--------------------------------------------")

    # print transitive closure
    for v in range(G.V()):
        print("{x:3d}: ".format(x=v), end='')
        for w in range(G.V()):
            if (tc.reachable(v, w)):
                print("  T", end='')
            else:
                print("   ", end='')
        print()
    

if __name__ == '__main__':
    main(sys.argv[1:])



# Created for BADS 2018
# see README.md for details
# This is python3 

from itu.algs4.fundamentals.queue import Queue


class BinarySearchST:
    """
    The BST class represents an ordered symbol table of generic
    key-value pairs.
    It supports the usual put, get, contains,
    delete, size, and is-empty methods.
    It also provides ordered methods for finding the minimum,
    maximum, floor, select, and ceiling.
    It also provides a keys method for iterating over all of the keys.
    A symbol table implements the associative array abstraction:
    when associating a value with a key that is already in the symbol table,
    the convention is to replace the old value with the new value.
    Unlike java.util.Map, this class uses the convention that
    values cannot be None—setting the
    value associated with a key to None is equivalent to deleting the key
    from the symbol table.

    This implementation uses a sorted array. It requires that
    the key type implements the Comparable interface and calls the
    compareTo() and method to compare two keys. It does not call either
    equals() or  hashCode().
    The put and remove operations each take linear time in
    the worst case the contains, ceiling, floor,
    and rank operations take logarithmic time the size,
    is-empty, minimum, maximum, and select
    operations take constant time. Construction takes constant time.
    """
    _INIT_CAPACITY = 2

    def __init__(self, capacity = _INIT_CAPACITY):
        """
        Initializes an empty symbol table with the specified initial capacity.
        :param capacity: the maximum capacity        
        """
        self._keys = [None] * capacity
        self._vals = [None] * capacity
        self._n = 0
    
    def _resize(self, capacity):
        # resize the underlying "arrays"
        assert capacity >= self._n
        tempk = [None] * capacity
        tempv = [None] * capacity
        for i in range(self._n):
            tempk[i] = self._keys[i]
            tempv[i] = self._vals[i]
        
        self._vals = tempv
        self._keys = tempk
    
    def size(self):
        """
        Returns the number of key-value pairs in this symbol table.
        
        :returns: the number of key-value pairs in this symbol table
        """
        return self._n

    def __len__(self):
        return self.size()
    
    def is_empty(self):
        """
        Returns True if this symbol table is empty.
        
        :returns: True if this symbol table is empty
                False otherwise
        """
        return self.size() == 0

    def contains(self, key):
        """
        Does this symbol table contain the given key?

        :param key: the key
        :returns:  True if this symbol table contains  key and
                False otherwise
        :raises ValueError: if  key is  None
        """
        if key is None: raise ValueError("argument to contains() is None")
        return self.get(key) is not None

    def get(self, key):
        """
        Returns the value associated with the given key in this symbol table.
        
        :param key: the key
        :returns: the value associated with the given key if the key is in the symbol table
            and None if the key is not in the symbol table
        :raises ValueError: if key is None
        """
        if key is None: raise ValueError("argument to get() is None") 
        if self.is_empty(): return None
        i = self.rank(key)
        if i < self._n and self._keys[i] == key: return self._vals[i]
        return None
    
    def rank(self, key):
        """
        Returns the number of keys in this symbol table strictly less than  key.
        
        :param key: the key
        :returns: the number of keys in the symbol table strictly less than  key
        :raises ValueError: if  key is  None
        """
        if key is None: raise ValueError("argument to rank() is None") 

        lo = 0
        hi = self._n-1 
        while lo <= hi: 
            mid = int(lo + (hi - lo) / 2)
            if key < self._keys[mid]: hi = mid - 1 
            elif key > self._keys[mid]: lo = mid + 1 
            else: return mid         
        return lo
    
    def put(self, key, val):
        """
        Inserts the specified key-value pair into the symbol table, overwriting the old 
        value with the new value if the symbol table already contains the specified key.
        Deletes the specified key (and its associated value) from this symbol table
        if the specified value is None.
        
        :param key: the key
        :param val: the value
        :raises ValueError: if  key is None
        """
        if key is None: raise ValueError("first argument to put() is None") 

        if val is None:
            self.delete(key)
            return
        
        i = self.rank(key)

        # key is already in table
        if i < self._n and self._keys[i] == key:
            self._vals[i] = val
            return        

        # insert new key-value pair
        if self._n == len(self._keys): self._resize(2*len(self._keys))

        j = self._n
        while j > i:
            self._keys[j] = self._keys[j-1]
            self._vals[j] = self._vals[j-1]
            j -= 1
        
        self._keys[i] = key
        self._vals[i] = val
        self._n += 1

        assert self._check()
     
    def delete(self, key):
        """
        Removes the specified key and associated value from this symbol table
        (if the key is in the symbol table).
        
        :param key: the key
        :raises ValueError: if  key is  None
        """
        if key is None: raise ValueError("argument to delete() is None") 
        if self.is_empty(): return

        # compute rank
        i = self.rank(key)
        n = self._n

        # key not in table
        if i == n or self._keys[i] != key:
            return

        j = i
        while j < self._n-1:
            self._keys[j] = self._keys[j+1]
            self._vals[j] = self._vals[j+1]
            j += 1

        self._n -= 1
        n = self._n
        self._keys[n] = None  # to avoid loitering
        self._vals[n] = None

        # resize if 1/4 full
        if n > 0 and n == len(self._keys)//4: self._resize(len(self._keys)//2)

        assert self._check()
    
    def deleteMin(self):
        """
        Removes the smallest key and associated value from this symbol table.
        
        :raises ValueError: if the symbol table is empty
        """
        if self.is_empty(): raise ValueError("Symbol table underflow error")
        self.delete(self.min())    

    def deleteMax(self):
        """
        Removes the largest key and associated value from this symbol table.
        
        :raises ValueError: if the symbol table is empty
        """
        if self.is_empty(): raise ValueError("Symbol table underflow error")
        self.delete(self.max())
    
   #*************************************************************************
   #                    Ordered symbol table methods.
   #*************************************************************************
    
    def min(self):
        """
        Returns the smallest key in this symbol table.
        
        :returns: the smallest key in this symbol table
        :raises ValueError: if this symbol table is empty
        """
        if self.is_empty(): raise ValueError("called min() with empty symbol table")
        return self._keys[0]
    

    def max(self):
        """
        Returns the largest key in this symbol table.
        
        :returns: the largest key in this symbol table
        :raises ValueError: if this symbol table is empty
        """
        if self.is_empty(): raise ValueError("called max() with empty symbol table")
        return self._keys[self._n-1]
   
    def select(self, k):
        """
        Return the kth smallest key in this symbol table.
        
        :param k: the order statistic
        :returns: the  kth smallest key in this symbol table
        :raises ValueError: unless k is between 0 and n-1
        """
        if k < 0 or k >= self.size():
            raise ValueError("called select() with invalid argument: {}".format(k))
        
        return self._keys[k]
    
    def floor(self, key):
        """
        Returns the largest key in this symbol table less than or equal to  key.
        
        :param key: the key
        :returns: the largest key in this symbol table less than or equal to  key
        :raises ValueError: if there is no such key
        :raises ValueError: if  key is  None
        """
        if key is None: raise ValueError("argument to floor() is None") 
        i = self.rank(key)
        if i < self._n and key == self._keys[i]: return self._keys[i]
        if i == 0: return None
        else: return self._keys[i-1]
    

    
    def ceiling(self, key):
        """
        Returns the smallest key in this symbol table greater than or equal to key.
        
        :param key: the key
        :returns: the smallest key in this symbol table greater than or equal to key
        :raises ValueError: if there is no such key
        :raises ValueError: if key is None
        """
        if key is None: raise ValueError("argument to ceiling() is None") 
        i = self.rank(key)
        if i == self._n: return None 
        else: return self._keys[i]
    
    def size_between(self, lo, hi):
        """
        Returns the number of keys in this symbol table in the specified range.
        
        :param lo: minimum endpoint
        :param hi: maximum endpoint
        :returns: the number of keys in this symbol table between lo 
            (inclusive) and  hi (inclusive)
        :raises ValueError: if either lo or hi is None
        """
        if lo is None: raise ValueError("first argument to size() is None") 
        if hi is None: raise ValueError("second argument to size() is None") 

        if lo > hi: return 0
        if self.contains(hi): return self.rank(hi) - self.rank(lo) + 1
        else:                 return self.rank(hi) - self.rank(lo)
   
    def keys(self):
        """
        Returns all keys in this symbol table as an  Iterable.
        To iterate over all of the keys in the symbol table named  st,
        use the foreach notation: for (Key key : st.keys()).
        
        :returns: all keys in this symbol table
        """
        return self.keys_between(self.min(), self.max())
    
    def keys_between(self, lo, hi):
        """
        Returns all keys in this symbol table in the given range,
        as an  Iterable.
        
        :param lo: minimum endpoint
        :param hi: maximum endpoint
        :returns: all keys in this symbol table between lo 
            (inclusive) and hi (inclusive)
        :raises ValueError: if either lo or hi are None
        """
        if lo is None: raise ValueError("first argument to keys() is None") 
        if hi is None: raise ValueError("second argument to keys() is None") 

        queue = Queue() 
        if lo > hi: return queue

        i = self.rank(lo)
        end = self.rank(hi)
        while i < end:
            queue.enqueue(self._keys[i])
            i += 1

        if self.contains(hi): queue.enqueue(self._keys[self.rank(hi)])
        return queue 

   #*************************************************************************
   #                    Check internal invariants.
   #*************************************************************************

    def _check(self):
        return self._is_sorted() and self._rank_check()
    
    
    def _is_sorted(self):
        # are the items in the array in ascending order?
        i = 1
        while i < self.size():
            if self._keys[i] < self._keys[i -1]: return False
            i += 1
        return True
    
    def _rank_check(self):
        # check that rank(select(i)) = i
        for i in range(self.size()):
            if i != self.rank(self.select(i)): return False
        for i in range(self.size()):
            if self._keys[i] != self.select(self.rank(self._keys[i])): return False
        return True
        
if __name__ == "__main__":
    from itu.algs4.stdlib import stdio

    st = BinarySearchST()
    i = 0
    while not stdio.isEmpty():
        key = stdio.readString()
        st.put(key, i)
        i += 1
        
    for s in st.keys():
        stdio.writef("%s %i\n", s, st.get(s))



class ThreeSum:
    @staticmethod
    def count(a):
        # Count triples that sum to 0
        n = len(a)
        count = 0
        for i in range(n):
            for j in range(i+1, n):
                for k in range(j+1, n):
                    if a[i] + a[j] + a[k] == 0:
                        count += 1
        return count



# Created for BADS 2018
# See README.md for details
# Python 3

"""
This module provides functions for sorting arrays of strings using lsd sort.

For additional documentation, see Section 5.1 of Algorithms, 4th Edition 
by Robert Sedgewick and Kevin Wayne.
"""

def sort(a, w, radix=256):
    """
     Rearranges the array of w-character strings in ascending order.
     
     :param a: the array to be sorted
     :param w: the number of characters per string
     :param radix: an optional number specifying the size of the alphabet to sort
    """
    
    n = len(a)
    aux = [None]*n
    
    for d in range (w-1, -1, -1): # from w-i to 0
        # sort by key-indexed counting on dth character
        
        # compute frequency counts
        count = [0]*(radix+1)
        for i in range(n):
            ch = ord(a[i][d]) # get number representation of character
            count[ch + 1] += 1
    
        # compute cumulates
        for r in range(radix):
            count[r+1] += count[r]
    
        # move data
        for i in range(n):
            ch = ord(a[i][d])
            aux[count[ch]] = a[i]
            count[ch] += 1
    
        # copy back
        for i in range(n):
            a[i] = aux[i]

import sys

from itu.algs4.stdlib import stdio

if __name__ == '__main__':
    if len(sys.argv) > 1:
        try: 
            sys.stdin = open(sys.argv[1])
        except IOError:
            print("File not found, using standard input instead")
    
    a = stdio.readAllStrings()
    sort(a, 3)
    for elem in a:
        print(elem)    



# code based on https://introcs.cs.princeton.edu/python/code/stdlib-python.zip as downloaded in dec 2017

"""
picture.py

The picture module defines the Picture class.
"""

#-----------------------------------------------------------------------

import os
import sys

import color
import pygame
from itu.algs4.stdlib import stdarray

#-----------------------------------------------------------------------

_DEFAULT_WIDTH = 512
_DEFAULT_HEIGHT = 512

#-----------------------------------------------------------------------

class Picture:
    """
    A Picture object models an image.  It is initialized such that
    it has a given width and height and contains all black pixels.
    Subsequently you can load an image from a given JPG or PNG file.
    """
    def __init__(self, arg1=None, arg2=None):
        """
        If both arg1 and arg2 are None, then construct self such that
        it is all black with _DEFAULT_WIDTH and height _DEFAULT_HEIGHT.
        If arg1 is not None and arg2 is None, then construct self by
        reading from the file whose name is arg1.
        If neither arg1 nor arg2 is None, then construct self such that
        it is all black with width arg1 and and height arg2.
        """
        if (arg1 is None) and (arg2 is None):
            maxW = _DEFAULT_WIDTH
            maxH = _DEFAULT_HEIGHT
            self._surface = pygame.Surface((maxW, maxH))
            self._surface.fill((0, 0, 0))
        elif (arg1 is not None) and (arg2 is None):
            fileName = arg1
            try:
                self._surface = pygame.image.load(fileName)
            except pygame.error:
                raise IOError()
        elif (arg1 is not None) and (arg2 is not None):
            maxW = arg1
            maxH = arg2
            self._surface = pygame.Surface((maxW, maxH))
            self._surface.fill((0, 0, 0))
        else:
            raise ValueError()
            
    #-------------------------------------------------------------------

    #def load(self, f):
    #    """
    #    Change self by reading from the file whose name is f. The
    #    dimensions of the read image override the dimensions specified
    #    in the constructor.
    #    """
        #if sys.hexversion >= 0x03000000:
        #    # Hack because Pygame without full image support
        #    # can handle only .bmp files.
        #    bmpFileName = f + '.bmp'
        #    os.system('convert ' + f + ' ' + bmpFileName)
        #    self._surface = pygame.image.load(bmpFileName)
        #    os.system('rm ' + bmpFileName)
        #else:
        #    self._surface = pygame.image.load(f)

    #    self._surface = pygame.image.load(f)

    #-------------------------------------------------------------------

    def save(self, f):
        """
        Save self to the file whose name is f.
        """
        #if sys.hexversion >= 0x03000000:
        #    # Hack because Pygame without full image support
        #    # can handle only .bmp files.
        #    bmpFileName = f + '.bmp'
        #    pygame.image.save(self._surface, bmpFileName)
        #    os.system('convert ' + bmpFileName + ' ' + f)
        #    os.system('rm ' + bmpFileName)
        #else:
        #    pygame.image.save(self._surface, f)

        pygame.image.save(self._surface, f)

    #-------------------------------------------------------------------

    def width(self):
        """
        Return the width of self.
        """
        return self._surface.get_width()

    #-------------------------------------------------------------------

    def height(self):
        """
        Return the height of self.
        """
        return self._surface.get_height()

    #-------------------------------------------------------------------

    def get(self, x, y):
        """
        Return the color of self at location (x, y).
        """
        pygameColor = self._surface.get_at((x, y))
        return color.Color(pygameColor.r, pygameColor.g, pygameColor.b)

    #-------------------------------------------------------------------

    def set(self, x, y, c):
        """
        Set the color of self at location (x, y) to c.
        """
        pygameColor = pygame.Color(c.getRed(), c.getGreen(),
           c.getBlue(), 0)
        self._surface.set_at((x, y), pygameColor)
